@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * dcraw_api.cc - API for DCRaw
- * Copyright 2004-2008 by Udi Fuchs
+ * Copyright 2004-2009 by Udi Fuchs
  *
  * based on dcraw by Dave Coffin
  * http://www.cybercom.net/~dcoffin/
@@ -37,13 +37,13 @@ extern "C" {
 int fc_INDI (const unsigned filters, const int row, const int col);
 void wavelet_denoise_INDI(gushort (*image)[4], const int black,
     const int iheight, const int iwidth, const int height, const int width,
-    const int colors, const int shrink, float pre_mul[4],
-    const float threshold, const unsigned filters, void *dcraw);
+    const int colors, const int shrink, const float pre_mul[4],
+    const float threshold, const unsigned filters);
 void scale_colors_INDI(gushort (*image)[4], const int maximum, const int black,
     const int use_auto_wb, const int use_camera_wb, const float cam_mul[4],
-    const int iheight, const int iwidth, const int colors, float pre_mul[4],
-    const unsigned filters, /*const*/ gushort white[8][8],
-    const char *ifname_display, void *dcraw);
+    const unsigned iheight, const unsigned iwidth, const int colors,
+    float pre_mul[4], const unsigned filters, /*const*/ gushort white[8][8],
+    const int shrink, const char *ifname_display, void *dcraw);
 void lin_interpolate_INDI(gushort (*image)[4], const unsigned filters,
     const int width, const int height, const int colors, void *dcraw);
 void vng_interpolate_INDI(gushort (*image)[4], const unsigned filters,
@@ -222,6 +222,7 @@ int dcraw_load_raw(dcraw_data *h)
     d->ifpSize = ftell(d->ifp);
     fseek(d->ifp, d->data_offset, SEEK_SET);
     (d->*d->load_raw)();
+    if (d->data_error) d->lastStatus = DCRAW_ERROR;
     if (d->zero_is_bad) d->remove_zeroes();
     d->bad_pixels(NULL);
     if (d->is_foveon) {
@@ -241,6 +242,11 @@ int dcraw_load_raw(dcraw_data *h)
     if (h->colors==3) h->pre_mul[3] = 0;
     memcpy(h->cam_mul, d->cam_mul, sizeof d->cam_mul);
     memcpy(h->rgb_cam, d->rgb_cam, sizeof d->rgb_cam);
+    // set post_mul in case wavelet_denoise() is called and scale_colors()
+    // was never called.
+    memcpy(h->post_mul, h->pre_mul, sizeof h->post_mul);
+    if (h->post_mul[3] == 0)
+	h->post_mul[3] = h->colors < 4 ? h->post_mul[1] : 1;
 
     double rgb_cam_transpose[4][3];
     for (i=0; i<4; i++) for (j=0; j<3; j++)
@@ -356,7 +362,7 @@ int dcraw_finalize_shrink(dcraw_image_data *f, dcraw_data *hh,
 		    for (ci=0; ci<scale; ci++) {
 			cl = fc_INDI(f4, r*scale+ri, c*scale+ci);
 			sum[cl] += get_pixel(hh, dark,
-				r*scale+ri, c*scale+ci, cl, pixels);
+				(r*scale+ri)/2, (c*scale+ci)/2, cl, pixels);
 			count[cl]++;
 		    }
 		for (cl=0; cl<hh->raw.colors; cl++)
@@ -367,10 +373,10 @@ int dcraw_finalize_shrink(dcraw_image_data *f, dcraw_data *hh,
 	    }
 	}
     } else {
-	if (hh->filters!=0) scale /= 2;
 	/* I'm skiping the last row/column if it is not a full row/column */
-	f->height = h = hh->raw.height / scale;
-	f->width = w = hh->raw.width / scale;
+	f->height = h = hh->height / scale;
+	f->width = w = hh->width / scale;
+	if (hh->filters!=0) scale /= 2;
 	fujiWidth = ( (hh->fuji_width+hh->shrink) >> hh->shrink ) / scale;
 	f->image = (dcraw_image_type *)g_realloc(
 			f->image, h * w * sizeof(dcraw_image_type));
@@ -403,6 +409,7 @@ int dcraw_image_resize(dcraw_image_data *image, int size)
     int mul=size, div=MAX(image->height, image->width);
 
     if (mul > div) return DCRAW_ERROR;
+    if (mul == div) return DCRAW_SUCCESS;
     /* I'm skiping the last row/column if it is not a full row/column */
     h = image->height * mul / div;
     w = image->width * mul / div;
@@ -501,38 +508,25 @@ int dcraw_set_color_scale(dcraw_data *h, int useAutoWB, int useCameraWB)
 	scale_colors_INDI(h->raw.image,
 		h->rgbMax-h->black, h->black, useAutoWB, useCameraWB,
 		h->cam_mul, h->raw.height, h->raw.width, h->raw.colors,
-		h->post_mul, h->filters, d->white, d->ifname_display, d);
+		h->post_mul, h->filters, d->white, h->shrink, 
+		d->ifname_display, d);
     h->message = d->messageBuffer;
     return d->lastStatus;
 }
 
-int dcraw_wavelet_denoise(dcraw_data *h, float threshold)
+void dcraw_wavelet_denoise(dcraw_data *h, float threshold)
 {
-    DCRaw *d = (DCRaw *)h->dcraw;
-    g_free(d->messageBuffer);
-    d->messageBuffer = NULL;
-    d->lastStatus = DCRAW_SUCCESS;
-    memcpy(h->post_mul, h->pre_mul, sizeof h->post_mul);
     if (threshold)
 	wavelet_denoise_INDI(h->raw.image, h->black, h->raw.height,
-		h->raw.width, h->height, h->width, h->raw.colors, h->shrink,
-		h->post_mul, threshold, h->filters, d);
-    h->message = d->messageBuffer;
-    return d->lastStatus;
+		h->raw.width, h->height, h->width, h->colors, h->shrink,
+		h->post_mul, threshold, h->fourColorFilters);
 }
 
-int dcraw_wavelet_denoise_shrinked(dcraw_image_data *f,
-	dcraw_data *h, float threshold)
+void dcraw_wavelet_denoise_shrinked(dcraw_image_data *f, float threshold)
 {
-    DCRaw *d = (DCRaw *)h->dcraw;
-    g_free(d->messageBuffer);
-    d->messageBuffer = NULL;
-    d->lastStatus = DCRAW_SUCCESS;
     if (threshold)
 	wavelet_denoise_INDI(f->image, 0, f->height, f->width, 0, 0, 4, 0,
-		NULL, threshold, 0, d);
-    h->message = d->messageBuffer;
-    return d->lastStatus;
+		NULL, threshold, 0);
 }
 
 int dcraw_finalize_interpolate(dcraw_image_data *f, dcraw_data *h,
