@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * dcraw_api.h - API for DCRaw
- * Copyright 2004-2008 by Udi Fuchs
+ * Copyright 2004-2009 by Udi Fuchs
  *
  * based on dcraw by Dave Coffin
  * http://www.cybercom.net/~dcoffin/
@@ -70,9 +70,8 @@ int dcraw_image_resize(dcraw_image_data *image, int size);
 int dcraw_image_stretch(dcraw_image_data *image, double pixel_aspect);
 int dcraw_flip_image(dcraw_image_data *image, int flip);
 int dcraw_set_color_scale(dcraw_data *h, int useAutoWB, int useCameraWB);
-int dcraw_wavelet_denoise(dcraw_data *h, float threshold);
-int dcraw_wavelet_denoise_shrinked(dcraw_image_data *f,
-	dcraw_data *h, float threshold);
+void dcraw_wavelet_denoise(dcraw_data *h, float threshold);
+void dcraw_wavelet_denoise_shrinked(dcraw_image_data *f, float threshold);
 int dcraw_finalize_interpolate(dcraw_image_data *f, dcraw_data *h,
 	dcraw_data *dark, int interpolation, int smoothing, int rgbWB[4]);
 void dcraw_close(dcraw_data *h);
