@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * dcraw_indi.c - DCRaw functions made independent
- * Copyright 2004-2008 by Udi Fuchs
+ * Copyright 2004-2009 by Udi Fuchs
  *
  * based on dcraw by Dave Coffin
  * http://www.cybercom.net/~dcoffin/
@@ -114,16 +114,15 @@ static void CLASS hat_transform (float *temp, float *base, int st, int size, int
 
 void CLASS wavelet_denoise_INDI(ushort (*image)[4], const int black,
        const int iheight, const int iwidth, const int height, const int width,
-       const int colors, const int shrink, float pre_mul[4],
-       const float threshold, const unsigned filters, void *dcraw)
+       const int colors, const int shrink, const float pre_mul[4],
+       const float threshold, const unsigned filters)
 {
-  float *fimg, thold, mul[2], avg, diff;
+  float *fimg=0, /* *temp,*/ thold, mul[2], avg, diff;
   int /*scale=1,*/ size, lev, hpass, lpass, row, col, nc, c, i, wlast;
   ushort *window[4];
   static const float noise[] =
   { 0.8002,0.2735,0.1202,0.0585,0.0291,0.0152,0.0080,0.0044 };
 
-    (void)dcraw;
 //  dcraw_message (dcraw, DCRAW_VERBOSE,_("Wavelet denoising...\n")); /*UF*/
 
 /* Scaling is done somewhere else - NKBJ*/
@@ -131,52 +130,57 @@ void CLASS wavelet_denoise_INDI(ushort (*image)[4], const int black,
   while (maximum << scale < 0x10000) scale++;
   maximum <<= --scale;
   black <<= scale;
-#endif
-  size = iheight*iwidth;
-  if ((nc = colors) == 3 && filters) nc++;
-  fimg = (float *) malloc (nc * (size*3 + iheight + iwidth) * sizeof *fimg);
+  if ((size = iheight*iwidth) < 0x15550000)
+    fimg = (float *) malloc ((size*3 + iheight + iwidth) * sizeof *fimg);
   merror (fimg, "wavelet_denoise()");
+  temp = fimg + size*3;
+#else
+  size = iheight*iwidth;
+  float temp[iheight + iwidth];
+#endif
+  if ((nc = colors) == 3 && filters) nc++;
 #ifdef _OPENMP
 #pragma omp parallel for				\
   default(none)						\
-  shared(nc,image,size,fimg)				\
-  private(c,i,hpass,lev,lpass,row,col,thold)
+  shared(nc,image,size)					\
+  private(c,i,hpass,lev,lpass,row,col,thold,fimg,temp)
 #endif
-  for (c=0; c < nc; c++) {      /* denoise R,G1,B,G3 individually */
-    float *fimgc = fimg + c * (size*3 + iheight + iwidth);
-    float *temp = fimgc + size*3;
+  FORC(nc) {			/* denoise R,G1,B,G3 individually */
+    fimg = (float *) malloc (size*3 * sizeof *fimg);
     for (i=0; i < size; i++)
-      fimgc[i] = sqrt((unsigned) (image[i][c] << (/*scale+*/16)));
+      fimg[i] = 256 * sqrt(image[i][c] /*<< scale*/);
     for (hpass=lev=0; lev < 5; lev++) {
       lpass = size*((lev & 1)+1);
       for (row=0; row < iheight; row++) {
-	hat_transform (temp, fimgc+hpass+row*iwidth, 1, iwidth, 1 << lev);
-	for (col=0; col < iwidth; col++)
-	  fimgc[lpass + row*iwidth + col] = temp[col] * 0.25;
+	hat_transform (temp, fimg+hpass+row*iwidth, 1, iwidth, 1 << lev);
+        for (col=0; col < iwidth; col++)
+	  fimg[lpass + row*iwidth + col] = temp[col] * 0.25;
       }
       for (col=0; col < iwidth; col++) {
-	hat_transform (temp, fimgc+lpass+col, iwidth, iheight, 1 << lev);
+	hat_transform (temp, fimg+lpass+col, iwidth, iheight, 1 << lev);
 	for (row=0; row < iheight; row++)
-	  fimgc[lpass + row*iwidth + col] = temp[row] * 0.25;
+	  fimg[lpass + row*iwidth + col] = temp[row] * 0.25;
       }
       thold = threshold * noise[lev];
       for (i=0; i < size; i++) {
-	fimgc[hpass+i] -= fimgc[lpass+i];
-	if      (fimgc[hpass+i] < -thold) fimgc[hpass+i] += thold;
-	else if (fimgc[hpass+i] >  thold) fimgc[hpass+i] -= thold;
-	else     fimgc[hpass+i] = 0;
-	if (hpass) fimgc[i] += fimgc[hpass+i];
+	fimg[hpass+i] -= fimg[lpass+i];
+	if	(fimg[hpass+i] < -thold) fimg[hpass+i] += thold;
+	else if (fimg[hpass+i] >  thold) fimg[hpass+i] -= thold;
+	else	 fimg[hpass+i] = 0;
+	if (hpass) fimg[i] += fimg[hpass+i];
       }
       hpass = lpass;
     }
     for (i=0; i < size; i++)
-      image[i][c] = CLIP((ushort)(SQR(fimgc[i]+fimgc[lpass+i])/0x10000));
+      image[i][c] = CLIP((ushort)(SQR(fimg[i]+fimg[lpass+i])/0x10000));
+    free(fimg);
   }
   if (filters && colors == 3) {  /* pull G1 and G3 closer together */
     for (row=0; row < 2; row++)
       mul[row] = 0.125 * pre_mul[FC(row+1,0) | 1] / pre_mul[FC(row,0) | 1];
+    ushort window_mem[4][width];
     for (i=0; i < 4; i++)
-      window[i] = (ushort *) fimg + width*i;
+      window[i] = window_mem[i]; /*(ushort *) fimg + width*i;*/
     for (wlast=-1, row=1; row < height-1; row++) {
       while (wlast < row+1) {
 	for (wlast++, i=0; i < 4; i++)
@@ -198,38 +202,55 @@ void CLASS wavelet_denoise_INDI(ushort (*image)[4], const int black,
       }
     }
   }
+#if 0
   free (fimg);
+#endif
 }
 
 void CLASS scale_colors_INDI(ushort (*image)[4], int maximum, const int black,
        const int use_auto_wb, const int use_camera_wb, const float cam_mul[4],
-       const int iheight, const int iwidth, const int colors, float pre_mul[4],
-       const unsigned filters, /*const*/ ushort white[8][8],
-       const char *ifname_display, void *dcraw)
+       const unsigned iheight, const unsigned iwidth, const int colors,
+       float pre_mul[4], const unsigned filters, /*const*/ ushort white[8][8],
+       const int shrink, const char *ifname_display, void *dcraw)
 {
-  int dblack, row, col, x, y, c, val, sum[8];
+  unsigned /*bottom, right, size,*/ row, col, /*ur, uc, i,*/ x, y, c, sum[8];
+  int val, dark, sat;
   double dsum[8], dmin, dmax;
-  float scale_mul[4];
+  float scale_mul[4]/*, fr, fc*/;
+#if 0
+  ushort *img=0, *pix;
 
+  if (user_mul[0])
+    memcpy (pre_mul, user_mul, sizeof pre_mul);
+#endif
   if (use_auto_wb || (use_camera_wb && cam_mul[0] == -1)) {
     memset (dsum, 0, sizeof dsum);
+#if 0
+    bottom = MIN (greybox[1]+greybox[3], height);
+    right  = MIN (greybox[0]+greybox[2], width);
+    for (row=greybox[1]; row < bottom; row += 8)
+      for (col=greybox[0]; col < right; col += 8) {
+#else
     for (row=0; row < iheight-7; row += 8)
       for (col=0; col < iwidth-7; col += 8) {
-       memset (sum, 0, sizeof sum);
-       for (y=row; y < row+8; y++)
-	 for (x=col; x < col+8; x++)
-	   FORC4 {
-	     val = image[y*iwidth+x][c];
-	     if (!val) continue;
-	     if (val > maximum-25) goto skip_block;
-	     val -= black;
-	     if (val < 0) val = 0;
-	     sum[c] += val;
-	     sum[c+4]++;
-	   }
-       for (c=0; c < 8; c++) dsum[c] += sum[c];
-skip_block:
-       continue;
+#endif
+	memset (sum, 0, sizeof sum);
+	for (y=row; y < row+8 /*&& y < bottom*/; y++)
+	  for (x=col; x < col+8 /*&& x < right*/; x++)
+	    FORC4 {
+	      if (filters) {
+		c = FC(y,x);
+		val = BAYER(y,x);
+	      } else
+		val = image[y*iwidth+x][c];
+	      if (val > maximum-25) goto skip_block;
+	      if ((val -= black) < 0) val = 0;
+	      sum[c] += val;
+	      sum[c+4]++;
+	      if (filters) break;
+	    }
+	FORC(8) dsum[c] += sum[c];
+skip_block: ;
       }
     FORC4 if (dsum[c]) pre_mul[c] = dsum[c+4] / dsum[c];
   }
@@ -251,11 +272,12 @@ skip_block:
       dcraw_message (dcraw, DCRAW_NO_CAMERA_WB,
 	      _("%s: Cannot use camera white balance.\n"), ifname_display);
   }
-//  if (user_mul[0])
-//    memcpy (pre_mul, user_mul, sizeof pre_mul);
   if (pre_mul[3] == 0) pre_mul[3] = colors < 4 ? pre_mul[1] : 1;
-  dblack = black;
-//  if (threshold) wavelet_denoise();
+  dark = black;
+  sat = maximum;
+#if 0
+  if (threshold) wavelet_denoise();
+#endif
   maximum -= black;
   for (dmin=DBL_MAX, dmax=c=0; c < 4; c++) {
     if (dmin > pre_mul[c])
@@ -263,24 +285,51 @@ skip_block:
     if (dmax < pre_mul[c])
 	dmax = pre_mul[c];
   }
-//  if (!highlight) dmax = dmin;
+#if 0
+  if (!highlight) dmax = dmin;
+#endif
   FORC4 scale_mul[c] = (pre_mul[c] /= dmax) * 65535.0 / maximum;
-  dcraw_message(dcraw, DCRAW_VERBOSE,_("Scaling with black %d, multipliers"),
-	  dblack);
+  dcraw_message(dcraw, DCRAW_VERBOSE,
+      _("Scaling with darkness %d, saturation %d, and\nmultipliers"),
+      dark, sat);
   FORC4 dcraw_message(dcraw, DCRAW_VERBOSE, " %f", pre_mul[c]);
   dcraw_message(dcraw, DCRAW_VERBOSE, "\n");
 
 /* The rest of the scaling is done somewhere else UF*/
 #if 0
-  for (row=0; row < iheight; row++)
-    for (col=0; col < iwidth; col++)
-      FORC4 {
-	val = image[row*iwidth+col][c];
-	if (!val) continue;
-	val -= black;
-	val = (int)(val*scale_mul[c]);
-	image[row*iwidth+col][c] = CLIP(val);
+  size = iheight*iwidth;
+  for (i=0; i < size*4; i++) {
+    val = image[0][i];
+    if (!val) continue;
+    val -= black;
+    val = (int)(val*scale_mul[i & 3]);
+    image[0][i] = CLIP(val);
+  }
+  if ((aber[0] != 1 || aber[2] != 1) && colors == 3) {
+    dcraw_message (DCRAW_VERBOSE,_("Correcting chromatic aberration...\n"));
+    for (c=0; c < 4; c+=2) {
+      if (aber[c] == 1) continue;
+      img = (ushort *) malloc (size * sizeof *img);
+      merror (img, "scale_colors()");
+      for (i=0; i < size; i++)
+	img[i] = image[i][c];
+      for (row=0; row < iheight; row++) {
+	fr = (row - iheight*0.5) * aber[c] + iheight*0.5; ur = (unsigned)fr;
+	if (ur > (unsigned)(iheight-2)) continue;
+	fr -= ur;
+	for (col=0; col < iwidth; col++) {
+	  fc = (col - iwidth*0.5) * aber[c] + iwidth*0.5; uc = (unsigned)fc;
+	  if (uc > (unsigned)(iwidth-2)) continue;
+	  fc -= uc;
+	  pix = img + ur*iwidth + uc;
+	  image[row*iwidth+col][c] = (ushort)(
+	    (pix[     0]*(1-fc) + pix[       1]*fc) * (1-fr) +
+	    (pix[iwidth]*(1-fc) + pix[iwidth+1]*fc) * fr);
+	}
       }
+      free(img);
+    }
+  }
 #endif
 }
 
@@ -358,7 +407,7 @@ void CLASS lin_interpolate_INDI(ushort (*image)[4], const unsigned filters,
 
    "Interpolation using a Threshold-based variable number of gradients"
 
-   described in http://www-ise.stanford.edu/~tingchen/algodep/vargra.html
+   described in http://scien.stanford.edu/class/psych221/projects/99/tingchen/algodep/vargra.html
 
    I've extended the basic idea to work with non-Bayer filter arrays.
    Gradients are numbered clockwise from NW=0 to W=7.
@@ -402,7 +451,7 @@ void CLASS vng_interpolate_INDI(ushort (*image)[4], const unsigned filters,
   if (filters == 1) prow = pcol = 15;
   int *ipalloc = ip = (int *) calloc ((prow+1)*(pcol+1), 1280);
   merror (ip, "vng_interpolate()");
-  for (row=0; row <= prow; row++)               /* Precalculate for VNG */
+  for (row=0; row <= prow; row++)		/* Precalculate for VNG */
     for (col=0; col <= pcol; col++) {
       code[row][col] = ip;
       for (cp=terms, t=0; t < 64; t++) {
@@ -433,66 +482,68 @@ void CLASS vng_interpolate_INDI(ushort (*image)[4], const unsigned filters,
       }
     }
 #ifdef _OPENMP
-#pragma omp parallel \
-    default(none) \
-    shared(image,code,prow,pcol) \
+#pragma omp parallel					\
+    default(none)					\
+    shared(image,code,prow,pcol)			\
     private(row,col,g,brow,rowtmp,pix,ip,gval,diff,gmin,gmax,thold,sum,color,num,c,t)
 #endif
   {
-  int slice = (height - 4) / uf_omp_get_num_threads();
-  int start_row = 2 + slice * uf_omp_get_thread_num();
-  int end_row = MIN(start_row + slice, height - 2);
-  for (row=start_row; row < end_row; row++) {	/* Do VNG interpolation */
-    for (g = 0; g < 4; g++)
-      brow[g] = &rowtmp[(row + g - 2) % 4];
-    for (col=2; col < width-2; col++) {
-      pix = image[row*width+col];
-      ip = code[row & prow][col & pcol];
-      memset (gval, 0, sizeof gval);
-      while ((g = ip[0]) != INT_MAX) {		/* Calculate gradients */
-	diff = ABS(pix[g] - pix[ip[1]]) << ip[2];
-	gval[ip[3]] += diff;
-	ip += 5;
-	if ((g = ip[-1]) == -1) continue;
-	gval[g] += diff;
-	while ((g = *ip++) != -1)
+    int slice = (height - 4) / uf_omp_get_num_threads();
+    int start_row = 2 + slice * uf_omp_get_thread_num();
+    int end_row = MIN(start_row + slice, height - 2);
+    for (row=start_row; row < end_row; row++) {	/* Do VNG interpolation */
+      for (g = 0; g < 4; g++)
+	brow[g] = &rowtmp[(row + g - 2) % 4];
+      for (col=2; col < width-2; col++) {
+	pix = image[row*width+col];
+	ip = code[row & prow][col & pcol];
+	memset (gval, 0, sizeof gval);
+	while ((g = ip[0]) != INT_MAX) {	/* Calculate gradients */
+	  diff = ABS(pix[g] - pix[ip[1]]) << ip[2];
+	  gval[ip[3]] += diff;
+	  ip += 5;
+	  if ((g = ip[-1]) == -1) continue;
 	  gval[g] += diff;
-      }
-      ip++;
-      gmin = gmax = gval[0];			/* Choose a threshold */
-      for (g=1; g < 8; g++) {
-	if (gmin > gval[g]) gmin = gval[g];
-	if (gmax < gval[g]) gmax = gval[g];
-      }
-      if (gmax == 0) {
-	memcpy (brow[2][col], pix, sizeof *image);
-	continue;
-      }
-      thold = gmin + (gmax >> 1);
-      memset (sum, 0, sizeof sum);
-      color = fc_INDI(filters,row,col);
-      for (num=g=0; g < 8; g++,ip+=2) {		/* Average the neighbors */
-	if (gval[g] <= thold) {
-	  FORCC
-	    if (c == color && ip[1])
-	      sum[c] += (pix[c] + pix[ip[1]]) >> 1;
-	    else
-	      sum[c] += pix[ip[0] + c];
-	  num++;
+	  while ((g = *ip++) != -1)
+	    gval[g] += diff;
+	}
+	ip++;
+	gmin = gmax = gval[0];			/* Choose a threshold */
+	for (g=1; g < 8; g++) {
+	  if (gmin > gval[g]) gmin = gval[g];
+	  if (gmax < gval[g]) gmax = gval[g];
+	}
+	if (gmax == 0) {
+	  memcpy (brow[2][col], pix, sizeof *image);
+	  continue;
+	}
+	thold = gmin + (gmax >> 1);
+	memset (sum, 0, sizeof sum);
+	color = fc_INDI(filters,row,col);
+	for (num=g=0; g < 8; g++,ip+=2) {	/* Average the neighbors */
+	  if (gval[g] <= thold) {
+	    FORCC
+	      if (c == color && ip[1])
+		sum[c] += (pix[c] + pix[ip[1]]) >> 1;
+	      else
+		sum[c] += pix[ip[0] + c];
+	    num++;
+	  }
+	}
+	FORCC {					/* Save to buffer */
+	  t = pix[color];
+	  if (c != color)
+	    t += (sum[c] - sum[color]) / num;
+	  brow[2][col][c] = CLIP(t);
 	}
       }
-      FORCC {					/* Save to buffer */
-	t = pix[color];
-	if (c != color)
-	  t += (sum[c] - sum[color]) / num;
-	brow[2][col][c] = CLIP(t);
-      }
+      if (row > start_row+1)			/* Write buffer to image */
+	memcpy (image[(row-2)*width+2], brow[0]+2, (width-4)*sizeof *image);
     }
-    if (row > start_row+1)			/* Write buffer to image */
+    if (row == height - 2) {
       memcpy (image[(row-2)*width+2], brow[0]+2, (width-4)*sizeof *image);
-  }
-  memcpy (image[(row-2)*width+2], brow[0]+2, (width-4)*sizeof *image);
-  memcpy (image[(row-1)*width+2], brow[1]+2, (width-4)*sizeof *image);
+      memcpy (image[(row-1)*width+2], brow[1]+2, (width-4)*sizeof *image);
+    }
   }
   free(ipalloc);
 }
@@ -500,76 +551,43 @@ void CLASS vng_interpolate_INDI(ushort (*image)[4], const unsigned filters,
 /*
    Patterned Pixel Grouping Interpolation by Alain Desbiolles
 */
-#define UT(c1, c2, c3, g1, g3) \
-  CLIP((long)(((g1 +g3) >> 1) +((c2-c1 +c2-c3) >> 3)))
-
-#define UT1(v1, v2, v3, c1, c3) \
-  CLIP((long)(v2 +((c1 +c3 -v1 -v3) >> 1)))
-
 void CLASS ppg_interpolate_INDI(ushort (*image)[4], const unsigned filters,
 	const int width, const int height, const int colors, void *dcraw)
 {
-  ushort (*pix)[4];            // Pixel matrix
-  ushort g2, c1, c2, cc1, cc2; // Simulated green and color
-  int    row, col, diff[2], guess[2], c, d, i;
-  int    dir[5]  = { 1, width, -1, -width, 1 };
-  int    g[2][4] = {{ -1 -2*width, -1 +2*width,  1 -2*width, 1 +2*width },
-		    { -2 -width,    2 -width,   -2 +width,   2 +width   }};
+  int dir[5] = { 1, width, -1, -width, 1 };
+  int row, col, diff[2], guess[2], c, d, i;
+  ushort (*pix)[4];
 
-  border_interpolate_INDI (height, width, image, filters, colors, 4);
+  border_interpolate_INDI (height, width, image, filters, colors, 3);
   dcraw_message (dcraw, DCRAW_VERBOSE,_("PPG interpolation...\n")); /*UF*/
 
 #ifdef _OPENMP
-#pragma omp parallel						\
-  default(none)							\
-  shared(image,dir,g)						\
-  private(row,col,i,d,c,pix,diff,guess,g2,c1,c2,cc1,cc2)
+#pragma omp parallel					\
+  default(none)						\
+  shared(image,dir)					\
+  private(row,col,i,d,c,pix,diff,guess)
 #endif
   {
-  // Fill in the green layer with gradients from RGB color pattern simulation
+/*  Fill in the green layer with gradients and pattern recognition: */
 #ifdef _OPENMP
 #pragma omp for
 #endif
-  for (row=3; row < height-4; row++) {
-    for (col=3+(FC(row,3) & 1), c=FC(row,col); col < width-4; col+=2) {
+  for (row=3; row < height-3; row++)
+    for (col=3+(FC(row,3) & 1), c=FC(row,col); col < width-3; col+=2) {
       pix = image + row*width+col;
-
-      // Horizontaly and verticaly
-      for (i=0; d=dir[i], i < 2; i++) {
-
-	// Simulate RGB color pattern
-	guess[i] = UT (pix[-2*d][c], pix[0][c], pix[2*d][c],
-		       pix[-d][1], pix[d][1]);
-	g2       = UT (pix[0][c], pix[2*d][c], pix[4*d][c],
-		       pix[d][1], pix[3*d][1]);
-	c1       = UT1(pix[-2*d][1], pix[-d][1], guess[i],
-		       pix[-2*d][c], pix[0][c]);
-	c2       = UT1(guess[i], pix[d][1], g2,
-		       pix[0][c], pix[2*d][c]);
-	cc1      = UT (pix[g[i][0]][1], pix[-d][1], pix[g[i][1]][1],
-		       pix[-1-width][2-c], pix[1-width][2-c]);
-	cc2      = UT (pix[g[i][2]][1],  pix[d][1], pix[g[i][3]][1],
-		       pix[-1+width][2-c], pix[1+width][2-c]);
-
-	// Calculate gradient with RGB simulated color
-	diff[i]  = ((ABS(pix[-d][1] -pix[-3*d][1]) +
-		     ABS(pix[0][c]  -pix[-2*d][c]) +
-		     ABS(cc1        -cc2)          +
-		     ABS(pix[0][c]  -pix[2*d][c])  +
-		     ABS(pix[d][1]  -pix[3*d][1])) * 2 / 3) +
-		     ABS(guess[i]   -pix[-d][1])   +
-		     ABS(pix[0][c]  -c1)           +
-		     ABS(pix[0][c]  -c2)           +
-		     ABS(guess[i]   -pix[d][1]);
+      for (i=0; (d=dir[i]) > 0; i++) {
+	guess[i] = (pix[-d][1] + pix[0][c] + pix[d][1]) * 2
+		      - pix[-2*d][c] - pix[2*d][c];
+	diff[i] = ( ABS(pix[-2*d][c] - pix[ 0][c]) +
+		    ABS(pix[ 2*d][c] - pix[ 0][c]) +
+		    ABS(pix[  -d][1] - pix[ d][1]) ) * 3 +
+		  ( ABS(pix[ 3*d][1] - pix[ d][1]) +
+		    ABS(pix[-3*d][1] - pix[-d][1]) ) * 2;
       }
-
-      // Then, select the best gradient
-      d = dir[diff[0] > diff[1]];
-      pix[0][1] = ULIM(guess[diff[0] > diff[1]], pix[-d][1], pix[d][1]);
+      d = dir[i = diff[0] > diff[1]];
+      pix[0][1] = ULIM(guess[i] >> 2, pix[d][1], pix[-d][1]);
     }
-  }
-
-  // Calculate red and blue for each green pixel
+/*  Calculate red and blue for each green pixel:		*/
 #ifdef _OPENMP
 #pragma omp for
 #endif
@@ -577,11 +595,10 @@ void CLASS ppg_interpolate_INDI(ushort (*image)[4], const unsigned filters,
     for (col=1+(FC(row,2) & 1), c=FC(row,col+1); col < width-1; col+=2) {
       pix = image + row*width+col;
       for (i=0; (d=dir[i]) > 0; c=2-c, i++)
-	pix[0][c] = UT1(pix[-d][1], pix[0][1], pix[d][1],
-			pix[-d][c], pix[d][c]);
+	pix[0][c] = CLIP((pix[-d][c] + pix[d][c] + 2*pix[0][1]
+			- pix[-d][1] - pix[d][1]) >> 1);
     }
-
-  // Calculate blue for red pixels and vice versa
+/*  Calculate blue for red pixels and vice versa:		*/
 #ifdef _OPENMP
 #pragma omp for
 #endif
@@ -589,12 +606,16 @@ void CLASS ppg_interpolate_INDI(ushort (*image)[4], const unsigned filters,
     for (col=1+(FC(row,1) & 1), c=2-FC(row,col); col < width-1; col+=2) {
       pix = image + row*width+col;
       for (i=0; (d=dir[i]+dir[i+1]) > 0; i++) {
-	diff[i]  = ABS(pix[-d][c] - pix[d][c]) +
-		   ABS(pix[-d][1] - pix[d][1]);
-	guess[i] = UT1(pix[-d][1], pix[0][1], pix[d][1],
-		       pix[-d][c], pix[d][c]);
+	diff[i] = ABS(pix[-d][c] - pix[d][c]) +
+		  ABS(pix[-d][1] - pix[0][1]) +
+		  ABS(pix[ d][1] - pix[0][1]);
+	guess[i] = pix[-d][c] + pix[d][c] + 2*pix[0][1]
+		 - pix[-d][1] - pix[d][1];
       }
-      pix[0][c] = CLIP(guess[diff[0] > diff[1]]);
+      if (diff[0] != diff[1])
+	pix[0][c] = CLIP(guess[diff[0] > diff[1]] >> 1);
+      else
+	pix[0][c] = CLIP((guess[0]+guess[1]) >> 2);
     }
   }
 }
@@ -621,8 +642,8 @@ void CLASS ahd_interpolate_INDI(ushort (*image)[4], const unsigned filters,
   dcraw_message (dcraw, DCRAW_VERBOSE,_("AHD interpolation...\n")); /*UF*/
 
 #ifdef _OPENMP
-#pragma omp parallel							\
-  default(shared)							\
+#pragma omp parallel					\
+  default(shared)					\
   private(top, left, row, col, pix, rix, lix, c, xyz, val, d, tc, tr, i, j, k, ldiff, abdiff, leps, abeps, hm, buffer, rgb, lab, homo, r)
 #endif
   {
@@ -654,7 +675,7 @@ void CLASS ahd_interpolate_INDI(ushort (*image)[4], const unsigned filters,
   for (top=2; top < height-5; top += TS-6)
     for (left=2; left < width-5; left += TS-6) {
 
-/*  Interpolate green horizontally and vertically:              */
+/*  Interpolate green horizontally and vertically:		*/
       for (row = top; row < top+TS && row < height-2; row++) {
 	col = left + (FC(row,left) & 1);
 	for (c = FC(row,col); col < left+TS && col < width-2; col+=2) {
@@ -862,8 +883,7 @@ void CLASS fuji_rotate_INDI(ushort (**image_p)[4], int *height_p,
   merror (img, "fuji_rotate()");
 
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) \
-  private(row,col,ur,uc,r,c,fr,fc,pix,i)
+#pragma omp parallel for default(shared) private(row,col,ur,uc,r,c,fr,fc,pix,i)
 #endif
   for (row=0; row < high; row++)
     for (col=0; col < wide; col++) {
