@@ -1,7 +1,7 @@
 /***************************************************
  nikon_curve.c - read Nikon NTC/NCV files
 
- Copyright 2004-2008 by Shawn Freeman, Udi Fuchs
+ Copyright 2004-2009 by Shawn Freeman, Udi Fuchs
 
  This program reads in a Nikon NTC/NCV file,
  interperates it's tone curve, and writes out a
@@ -111,7 +111,7 @@ int ProcessArgs(int num_args, char *args[])
 	    printf("NOTE: If a resolution is not specified, a default one will be used.\n");
 	    printf("      If the -o option is not specified, default files will be used.\n\n");
 	    printf("Example:\n");
-	    printf("NikonCurveGenerator -sr 65536 -or 256 curveFile -o exportFile\n");
+	    printf("%s -sr 65536 -or 256 curveFile -o exportFile\n", args[0]);
 
 	    //signal that processing cannot occur
 	    return NC_ERROR;
@@ -254,7 +254,7 @@ size_t nc_fread(void *ptr, size_t size, size_t nmemb, FILE *stream)
 
 size_t nc_fwrite(const void *ptr, size_t size, size_t nmemb, FILE *stream)
 {
-    size_t num = nc_fwrite(ptr, size, nmemb, stream);
+    size_t num = fwrite(ptr, size, nmemb, stream);
     if ( num!=nmemb )
 	nc_message(NC_WARNING, "WARNING: nc_fwrite %d != %d\n", num, nmemb);
     return num;
@@ -272,12 +272,15 @@ isBigEndian:
 int isBigEndian()
 {
     STATIC_ASSERT(sizeof(short)==2);
-    short x;
-    unsigned char EndianTest[2] = { 1, 0 };
+    union {
+	unsigned char c[2];
+	short x;
+    } EndianTest;
 
-    x = *(short *) EndianTest;
+    EndianTest.c[0] = 1;
+    EndianTest.c[1] = 0;
 
-    return (x!=1);
+    return (EndianTest.x!=1);
 }
 
 /***********************************************************************
