@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw-batch.c - The standalone interface to UFRaw in batch mode.
- * Copyright 2004-2008 by Udi Fuchs
+ * Copyright 2004-2009 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -86,6 +86,8 @@ int main (int argc, char **argv)
 	    continue;
 	}
 	status = ufraw_config(uf, &rc, &conf, &cmd);
+	if (uf->conf && uf->conf->createID == only_id && cmd.createID!=only_id)
+	    uf->conf->createID = also_id;
 	if (status==UFRAW_ERROR) {
 	    exitCode = 1;
 	    ufraw_close(uf);
@@ -103,12 +105,15 @@ int main (int argc, char **argv)
 	    g_snprintf(stat, max_name, "[%d/%d]", fileIndex, fileCount);
 	else
 	    stat[0] = '\0';
-	ufraw_message(UFRAW_MESSAGE, _("Loaded %s%s"), uf->filename, stat);
-	if (ufraw_batch_saver(uf)==UFRAW_SUCCESS)
-	    ufraw_message(UFRAW_MESSAGE, _("Saved %s%s"),
-		    uf->conf->outputFilename, stat);
-	else
+	ufraw_message(UFRAW_MESSAGE, _("Loaded %s %s"), uf->filename, stat);
+	status = ufraw_batch_saver(uf);
+	if (status==UFRAW_SUCCESS || status==UFRAW_WARNING) {
+	    if (uf->conf->createID!=only_id)
+		ufraw_message(UFRAW_MESSAGE, _("Saved %s %s"),
+			uf->conf->outputFilename, stat);
+	} else {
 	    exitCode = 1;
+	}
 	ufraw_close(uf);
 	g_free(uf);
     }
