@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw-gimp.c - The GIMP plug-in.
- * Copyright 2004-2008 by Udi Fuchs
+ * Copyright 2004-2009 by Udi Fuchs
  *
  * based on the GIMP plug-in by Pawel T. Jochym jochym at ifj edu pl,
  *
@@ -88,7 +88,7 @@ void query()
 	    "Udi Fuchs",
 	    "Copyright 2003 by Dave Coffin\n"
 	    "Copyright 2004 by Pawel Jochym\n"
-	    "Copyright 2004-2008 by Udi Fuchs",
+	    "Copyright 2004-2009 by Udi Fuchs",
 	    "ufraw-" VERSION,
 #if GIMP_CHECK_VERSION(2,2,0)
 	    "raw image",
@@ -109,7 +109,7 @@ void query()
 	    "Loads thumbnails from digital camera raw files.",
 	    "Loads thumbnails from digital camera raw files.",
 	    "Udi Fuchs",
-	    "Copyright 2004-2008 by Udi Fuchs",
+	    "Copyright 2004-2009 by Udi Fuchs",
 	    "ufraw-" VERSION,
 	    NULL,
 	    NULL,
@@ -297,15 +297,25 @@ void run(GIMP_CONST gchar *name,
     }
 }
 
+int gimp_row_writer(ufraw_data *uf, void *volatile out, void *pixbuf,
+    int row, int width, int height, int grayscale, int bitDepth)
+{
+    (void)uf;
+    (void)grayscale;
+    (void)bitDepth;
+
+    gimp_pixel_rgn_set_rect(out, pixbuf, 0, row, width, height);
+
+    return UFRAW_SUCCESS;
+}
+
 long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
 {
     GimpDrawable *drawable;
     GimpPixelRgn pixel_region;
     gint32 layer;
-    guint8 *pixbuf;
-    guint16 *pixtmp;
-    int height, width, top, left, depth, tile_height, row, nrows, rowStride, y;
-    image_type *rawImage;
+    int height, width, top, left, depth, tile_height, row, nrows;
+    (void)widget;
 
     uf->gimpImage = -1;
 
@@ -320,12 +330,13 @@ long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
     } else {
 	if (ufraw_convert_image(uf)!=UFRAW_SUCCESS)
 	    return UFRAW_ERROR;
+	ufraw_image_data *FirstImage = &uf->Images[ufraw_first_phase];
 	height = (uf->conf->CropY2 - uf->conf->CropY1)
-		* uf->image.height / uf->initialHeight;
+		* FirstImage->height / uf->initialHeight;
 	width = (uf->conf->CropX2 - uf->conf->CropX1)
-		* uf->image.width / uf->initialWidth;
-	top = uf->conf->CropY1 * uf->image.height / uf->initialHeight;
-	left = uf->conf->CropX1 * uf->image.width / uf->initialWidth;
+		* FirstImage->width / uf->initialWidth;
+	top = uf->conf->CropY1 * FirstImage->height / uf->initialHeight;
+	left = uf->conf->CropX1 * FirstImage->width / uf->initialWidth;
 #ifdef UFRAW_CINEPAINT
 	if ( uf->conf->profile[out_profile]
 			[uf->conf->profileIndex[out_profile]].BitDepth==16 )
@@ -363,25 +374,8 @@ long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
 		    uf->thumb.buffer+3*row*width, 0, row, width, nrows);
 	}
     } else {
-	pixbuf = g_new(guint8, tile_height * width * depth);
-	pixtmp = g_new(guint16, tile_height * width * 3);
-	rowStride = uf->image.width;
-	rawImage = uf->image.image;
-	for (row = 0; row < height; row += tile_height) {
-	    if ( sendToGimpMode )
-		gimp_progress_update(0.5 + 0.5*row/height);
-	    else
-		preview_progress(widget, _("Loading image"),
-			0.5 + 0.5*row/height);
-	    nrows = MIN(height-row, tile_height);
-	    for (y=0 ; y<nrows; y++)
-		develope(&pixbuf[y*width*depth], rawImage[(top+row+y)*rowStride+left],
-			uf->developer, depth==3 ? 8 : 16, pixtmp, width);
-	    gimp_pixel_rgn_set_rect(&pixel_region, pixbuf, 0, row,
-		    width, nrows);
-	}
-	g_free(pixbuf);
-	g_free(pixtmp);
+	ufraw_write_image_data(uf, &pixel_region, width, height, left, top,
+		depth==3 ? 8 : 16, 0, gimp_row_writer);
     }
     gimp_drawable_flush(drawable);
     gimp_drawable_detach(drawable);
