@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw.c - The standalone interface to UFRaw.
- * Copyright 2004-2008 by Udi Fuchs
+ * Copyright 2004-2009 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -72,11 +72,6 @@ int main (int argc, char **argv)
 		_("--silent option is valid only in batch mode"));
 	optInd = -1;
     }
-    if ( cmd.rotate!=-1 ) {
-	ufraw_message(UFRAW_ERROR,
-		_("--rotate option is valid only in batch mode"));
-	optInd = -1;
-    }
     if ( cmd.embeddedImage ) {
 	ufraw_message(UFRAW_ERROR, _("Extracting embedded image is not supported in interactive mode"));
 	optInd = -1;
