@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw.h - Common definitions for UFRaw.
- * Copyright 2004-2008 by Udi Fuchs
+ * Copyright 2004-2009 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #include "nikon_curve.h"
 #include <time.h> // for time_t
+#include "dcraw_api.h"
 
 /* macro to clamp a number between two values */
 #ifndef LIM
@@ -32,6 +33,7 @@
 #define max_profiles 20
 #define max_path 200
 #define max_name 80
+#define max_adjustments 3
 
 /* An impossible value for conf float values */
 #define NULLF -10000.0
@@ -81,17 +83,24 @@ typedef struct {
     double channel[4];
 } wb_data;
 
+typedef struct {
+    double adjustment;
+    double hue;
+    double hueWidth;
+} lightness_adjustment;
+
 typedef struct {
     DeveloperMode mode;
     unsigned rgbMax, max, exposure, colors, useMatrix;
     int restoreDetails, clipHighlights;
-    int rgbWB[4], colorMatrix[3][4];
+    int doWB, rgbWB[4], colorMatrix[3][4];
     double gamma, linear;
     char profileFile[profile_types][max_path];
     void *profile[profile_types];
     Intent intent[profile_types];
     gboolean updateTransform;
     void *colorTransform;
+    void *rgbtolabTransform;
     double saturation;
 #ifdef UFRAW_CONTRAST
     double contrast;
@@ -101,8 +110,10 @@ typedef struct {
     void *luminosityProfile;
     void *TransferFunction[3];
     void *saturationProfile;
+    void *adjustmentProfile;
     GrayscaleMode grayscaleMode;
     double grayscaleMixer[3];
+    lightness_adjustment lightnessAdjustment[max_adjustments];
 } developer_data;
 
 typedef guint16 image_type[4];
@@ -140,7 +151,7 @@ typedef struct {
  * CONF|ID: curve/profile are added to the list from RC.
  * CONF: inputFilename, outputFilename are ignored.
  * outputPath can only be specified in CMD or guessed in interactive mode.
- * ID: createID==only_id is switched to also_id.
+ * ID: createID==only_id is switched to also_id in case of ufraw-batch.
  * ID: chanMul[] override wb, green, temperature.
  */
 typedef struct {
@@ -154,6 +165,9 @@ typedef struct {
     double temperature, green;
     double chanMul[4];
     double threshold;
+#ifdef UFRAW_HOTPIXELS
+    double hotpixel;
+#endif
 #ifdef UFRAW_CONTRAST
     double contrast;
 #endif
@@ -173,7 +187,10 @@ typedef struct {
     char darkframeFile[max_path];
     struct ufraw_struct *darkframe;
     int CropX1, CropY1, CropX2, CropY2;
+    int orientation;
     double rotationAngle;
+    int lightnessAdjustmentCount;
+    lightness_adjustment lightnessAdjustment[max_adjustments];
     int grayscaleMode;
     double grayscaleMixer[3];
 
@@ -188,7 +205,6 @@ typedef struct {
 
     /* GUI settings */
     double Zoom;
-    int Scale;
     gboolean LockAspect; /* True if aspect ratio is locked */
     int saveConfiguration;
     int histogram, liveHistogramScale;
@@ -197,13 +213,14 @@ typedef struct {
     gboolean overExp, underExp, blinkOverUnder;
     gboolean RememberOutputPath;
     gboolean WindowMaximized;
+    int drawLines;
     char curvePath[max_path];
     char profilePath[max_path];
     gboolean silent;
     char remoteGimpCommand[max_path];
 
     /* EXIF data */
-    int orientation;
+    int CameraOrientation;
     float iso_speed, shutter, aperture, focal_len, subject_distance;
     char exifSource[max_name], isoText[max_name], shutterText[max_name],
 	 apertureText[max_name], focalLenText[max_name],
@@ -237,13 +254,6 @@ typedef struct {
     long valid;
 } ufraw_image_data;
 
-/* image_data should be phased out and replaced by ufraw_image_data */
-typedef struct {
-    image_type *image;
-    guint8 *buffer;
-    int height, width, depth, rowstride;
-} image_data;
-
 typedef struct ufraw_struct {
     int status;
     char *message;
@@ -254,8 +264,7 @@ typedef struct ufraw_struct {
     float rgb_cam[3][4];
     int ConvertShrink;
     ufraw_image_data Images[ufraw_phases_num];
-    image_data image;
-    image_data thumb;
+    ufraw_image_data thumb;
     void *raw;
     gboolean HaveFilters;
     void *unzippedBuf;
@@ -275,6 +284,10 @@ typedef struct ufraw_struct {
 #ifdef HAVE_LENSFUN
     int postproc_ops; /* postprocessing operations (LF_MODIFY_XXX) */
     lfModifier *modifier;
+#endif /* HAVE_LENSFUN */
+#ifdef UFRAW_HOTPIXELS
+    int hotpixels;
+    gboolean mark_hotpixels;
 #endif
 } ufraw_data;
 
@@ -298,16 +311,21 @@ int ufraw_convert_image(ufraw_data *uf);
 int ufraw_convert_image_init(ufraw_data *uf);
 int ufraw_convert_image_first_phase(ufraw_data *uf, gboolean lensfix);
 int ufraw_convert_image_init_phase(ufraw_data *uf);
-ufraw_image_data *ufraw_convert_image_area (ufraw_data *uf, int saidx, UFRawPhase phase);
+ufraw_image_data *ufraw_convert_get_final_image(ufraw_data *uf);
+ufraw_image_data *ufraw_convert_image_area(ufraw_data *uf, unsigned saidx,
+	UFRawPhase phase);
 void ufraw_close(ufraw_data *uf);
-void ufraw_flip_orientation(ufraw_data *uf, int flip);
 int ufraw_flip_image(ufraw_data *uf, int flip);
 int ufraw_set_wb(ufraw_data *uf);
 void ufraw_auto_expose(ufraw_data *uf);
 void ufraw_auto_black(ufraw_data *uf);
 void ufraw_auto_curve(ufraw_data *uf);
-void ufraw_rotate_row(image_data *image, void *pixbuf, double angle,
+void ufraw_rotate_row(ufraw_image_data *image, void *pixbuf, double angle,
 		      int bitDepth, int row, int offset, int width);
+void ufraw_rotate_image_buffer(ufraw_image_data *img, double angle);
+void ufraw_normalize_rotation(ufraw_data *uf);
+void ufraw_unnormalize_rotation(ufraw_data *uf);
+void ufraw_get_image_dimensions(dcraw_data *raw, ufraw_data *uf);
 
 void ufraw_img_get_subarea_coord (ufraw_image_data *img, unsigned saidx,
                                   int *x, int *y, int *w, int *h);
@@ -369,6 +387,12 @@ int conf_set_cmd(conf_data *conf, const conf_data *cmd);
 int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc);
 
 /* prototype for functions in ufraw_developer.c */
+// Convert linear RGB to CIE-LCh
+void uf_rgb_to_cielch(gint64 rgb[3], float lch[3]);
+// Convert CIE-LCh to linear RGB
+void uf_cielch_to_rgb(float lch[3], gint64 rgb[3]);
+void uf_raw_to_cielch(const developer_data *d,
+		      const guint16 raw[4], float lch[3]);
 developer_data *developer_init();
 void developer_destroy(developer_data *d);
 void developer_profile(developer_data *d, int type, profile_data *p);
@@ -377,8 +401,7 @@ void developer_display_profile(developer_data *d,
 void developer_prepare(developer_data *d, conf_data *conf,
     int rgbMax, float rgb_cam[3][4], int colors, int useMatrix,
     DeveloperMode mode);
-void develope(void *po, guint16 pix[4], developer_data *d, int mode,
-    guint16 *buf, int count);
+void develop(void *po, guint16 pix[4], developer_data *d, int mode, int count);
 void develop_linear(guint16 in[4], guint16 out[3], developer_data *d);
 
 /* prototype for functions in ufraw_saver.c */
@@ -387,6 +410,11 @@ long ufraw_send_to_gimp(ufraw_data *uf);
 
 /* prototype for functions in ufraw_writer.c */
 int ufraw_write_image(ufraw_data *uf);
+void ufraw_write_image_data(
+    ufraw_data *uf,
+    void * volatile out,
+    int width, int height, int left, int top, int bitDepth, int grayscaleMode,
+    int (*row_writer) (ufraw_data *, void * volatile, void *, int, int, int, int, int));
 
 /* prototype for functions in ufraw_delete.c */
 long ufraw_delete(void *widget, ufraw_data *uf);
