@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_conf.c - handle configuration issues
- * Copyright 2004-2008 by Udi Fuchs
+ * Copyright 2004-2009 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,9 @@ const conf_data conf_default = {
     6500, 1.0, /* temperature, green */
     { -1.0, -1.0, -1.0, -1.0 }, /* chanMul[] */
     0.0, /* wavelet denoising threshold */
+#ifdef UFRAW_HOTPIXELS
+    0.0, /* hotpixel sensitivity */
+#endif
 #ifdef UFRAW_CONTRAST
     1.0, /* global contrast */
 #endif
@@ -63,12 +66,13 @@ const conf_data conf_default = {
       { N_("Linear curve"), TONE_CURVE, 0.0, 1.0, 0.0, 1.0, 1.0,
 	  2 , { { 0.0, 0.0 }, { 1.0, 1.0 } } }
     },
-    { 1, 0, 0 } , { 2, 1, 2 }, /* profileIndex[], profileCount[] */
+    { 1, 0, 0 } , { 2, 2, 2 }, /* profileIndex[], profileCount[] */
     /* Profile data defaults */
     { { { N_("No profile"), "", "", 0.45, 0.1, 0 },
 	{ N_("Color matrix"), "", "", 0.45, 0.1, 0 },
 	{ "Some ICC Profile", "", "", 0.45, 0.0, 0 } },
       { { N_("sRGB"), "", "", 0.0, 0.0, 8 },
+	{ N_("sRGB (embedded)"), "", "", 0.0, 0.0, 8 },
 	{ "Some ICC Profile", "", "", 0.0, 0.0, 8 } },
       { { N_("System default"), "", "", 0.0, 0.0, 0 },
 	{ N_("sRGB"), "", "", 0.0, 0.0, 0 },
@@ -77,7 +81,14 @@ const conf_data conf_default = {
     ahd_interpolation, 0, /* interpolation, smoothing */
     "", NULL, /* darkframeFile, darkframe */
     -1, -1, -1, -1, /* Crop X1,Y1,X2,Y2 */
+    -1, /* orientation */
     0, /* rotationAngle */
+    0, /* lightness adjustment count */
+    {
+	{ 1, 0, 0 },
+	{ 1, 0, 0 },
+	{ 1, 0, 0 },
+    }, /* lightness adjustments */
     grayscale_none, /* grayscale mode */
     { 1.0, 1.0, 1.0 }, /* grayscale mixer */
     /* Save options */
@@ -93,7 +104,7 @@ const conf_data conf_default = {
     TRUE, /* rotate to camera's setting */
 
     /* GUI settings */
-    25.0, 4, TRUE, /* Zoom, Scale, LockAspect */
+    25.0, TRUE, /* Zoom, LockAspect */
     enabled_state, /* saveConfiguration */
     rgb_histogram, /* histogram */
     linear_histogram, /* liveHistogramScale */
@@ -104,6 +115,7 @@ const conf_data conf_default = {
     TRUE, /* blinkOverUnder indicators */
     FALSE, /* RememberOutputPath */
     FALSE, /* WindowMaximized */
+    0, /* number of helper lines to draw */
     "", "", /* curvePath, profilePath */
     FALSE, /* silent */
 #ifdef WIN32
@@ -115,7 +127,7 @@ const conf_data conf_default = {
 #endif
 
     /* EXIF data */
-    -1, /* orientation */
+    0, /* CameraOrientation */
     0.0, 0.0, 0.0, 0.0, 1.0, /* iso_speed, shutter, aperture, focal_len, subject_dist */
     "", "", "", "", /* exifSource, isoText, shutterText, apertureText */
     "", "", "", "", /* focalLenText, focalLen35Text, lensText, flashText */
@@ -134,7 +146,7 @@ const conf_data conf_default = {
     { LF_VIGNETTING_MODEL_NONE, 0, 0, 0, { 0, 0, 0 } },
     LF_UNKNOWN,                   /* lens type */
     0,                            /* lens postprocessing scale power-of-two */ 
-#endif
+#endif /* HAVE_LENSFUN */
 };
 
 static const char *interpolationNames[] =
@@ -148,7 +160,6 @@ static const char *intentNames[] =
 static const char *grayscaleModeNames[] =
     { "none", "lightness", "luminance", "value", "mixer", NULL };
 
-
 void conf_init (conf_data *c)
 {
 #ifdef HAVE_LENSFUN
@@ -156,17 +167,17 @@ void conf_init (conf_data *c)
 
     if (!lensdb)
     {
-        /* Load lens database */
-        lensdb = lf_db_new ();
-        lf_db_load (lensdb);
+	/* Load lens database */
+	lensdb = lf_db_new ();
+	lf_db_load (lensdb);
     }
-#endif
+#endif /* HAVE_LENSFUN */
 
     *c = conf_default;
 
 #ifdef HAVE_LENSFUN
     c->lensdb = lensdb;
-#endif
+#endif /* HAVE_LENSFUN */
 }
 
 static int conf_find_name(const char name[],const char *namesList[], int notFound)
@@ -244,6 +255,8 @@ static void conf_parse_start(GMarkupParseContext *context,
 		c->profileIndex[in_profile] = 1;
 	    if (!strcmp("sRGBOutputProfile", element))
 		c->profileIndex[out_profile] = 0;
+	    if (!strcmp("sRGBEmbeddedOutputProfile", element))
+		c->profileIndex[out_profile] = 1;
 	    if (!strcmp("SystemDisplayProfile", element))
 		c->profileIndex[display_profile] = 0;
 	    if (!strcmp("sRGBDisplayProfile", element))
@@ -295,6 +308,8 @@ static void conf_parse_start(GMarkupParseContext *context,
 	c->profileCount[in_profile] = -1;
     if ( !strcmp("sRGBOutputProfile", element) )
 	c->profileCount[out_profile] = 0;
+    if ( !strcmp("sRGBEmbeddedOutputProfile", element) )
+	c->profileCount[out_profile] = -1;
     if ( !strcmp("SystemDisplayProfile", element) )
 	c->profileCount[display_profile] = 0;
     if ( !strcmp("sRGBDisplayProfile", element) )
@@ -346,6 +361,9 @@ static void conf_parse_end(GMarkupParseContext *context, const gchar *element,
     if ( !strcmp("sRGBOutputProfile", element) )
 	c->profileCount[out_profile] =
 		conf_default.profileCount[out_profile];
+    if ( !strcmp("sRGBEmbeddedOutputProfile", element) )
+	c->profileCount[out_profile] =
+		conf_default.profileCount[out_profile];
     if ( !strcmp("SystemDisplayProfile", element) )
 	c->profileCount[display_profile] =
 		conf_default.profileCount[display_profile];
@@ -465,7 +483,7 @@ static void conf_parse_text(GMarkupParseContext *context, const gchar *text,
 	    g_free(utf8);
 	}
 	if (!strcmp("ProductName", element))
-	    g_strlcpy(c->profile[out_profile][i].productName, temp, max_path);
+	    g_strlcpy(c->profile[out_profile][i].productName, temp, max_name);
 	if (!strcmp("BitDepth", element))
 	    sscanf(temp, "%d", &c->profile[out_profile][i].BitDepth);
 	return;
@@ -479,7 +497,7 @@ static void conf_parse_text(GMarkupParseContext *context, const gchar *text,
 	}
 	if (!strcmp("ProductName", element))
 	    g_strlcpy(c->profile[display_profile][i].productName, temp,
-		    max_path);
+		    max_name);
 	return;
     }
     if (!strcmp("BaseCurve", element)) {
@@ -574,6 +592,8 @@ static void conf_parse_text(GMarkupParseContext *context, const gchar *text,
     if (!strcmp("UnderExposure", element)) sscanf(temp, "%d", &c->underExp);
     if (!strcmp("BlinkOverUnder", element))
 	sscanf(temp, "%d", &c->blinkOverUnder);
+    if (!strcmp("DrawLines", element))
+	sscanf(temp, "%d", &c->drawLines);
     if (!strcmp("RememberOutputPath", element))
 	sscanf(temp, "%d", &c->RememberOutputPath);
     if (!strcmp("WindowMaximized", element))
@@ -608,6 +628,10 @@ static void conf_parse_text(GMarkupParseContext *context, const gchar *text,
     }
     if (!strcmp("WaveletDenoisingThreshold", element))
 	sscanf(temp, "%lf", &c->threshold);
+#ifdef UFRAW_HOTPIXELS
+    if (!strcmp("HotpixelSensitivity", element))
+	sscanf(temp, "%lf", &c->hotpixel);
+#endif
 #ifdef UFRAW_CONTRAST
     if (!strcmp("Contrast", element))
         sscanf(temp, "%lf", &c->contrast);
@@ -642,6 +666,18 @@ static void conf_parse_text(GMarkupParseContext *context, const gchar *text,
 	else c->intent[out_profile] = conf_find_name(temp, intentNames,
 		    conf_default.intent[out_profile]);
     }
+    if ( strcmp("LightnessAdjustment", element) == 0 ) {
+	if (c->lightnessAdjustmentCount < max_adjustments) {
+	    lightness_adjustment *a = &c->lightnessAdjustment[c->lightnessAdjustmentCount];
+	    sscanf(temp, "%lf %lf %lf",
+		   &a->adjustment, &a->hue, &a->hueWidth);
+	    c->lightnessAdjustmentCount++;
+	}
+	else {
+	    ufraw_message(UFRAW_SET_ERROR,
+			  _("Too many lightness adjustments in the ID file, ignored\n"));
+	}
+    }
     if ( strcmp("GrayscaleMode", element)==0 ) {
         c->grayscaleMode = (sscanf(temp, "%d", &i)==1)
 	  ? i
@@ -661,6 +697,7 @@ static void conf_parse_text(GMarkupParseContext *context, const gchar *text,
 		conf_default.intent[display_profile]);
     if (!strcmp("Make", element)) g_strlcpy(c->make, temp, max_name);
     if (!strcmp("Model", element)) g_strlcpy(c->model, temp, max_name);
+    if (!strcmp("Lens", element)) g_strlcpy(c->lensText, temp, max_name);
     if (!strcmp("DarkframeFile", element))
 	g_strlcpy(c->darkframeFile, temp, max_path);
     if (!strcmp("ProfilePath", element)) {
@@ -808,21 +845,6 @@ int conf_save(conf_data *c, char *IDFilename, char **confBuffer)
 	char *utf8=g_filename_display_name(c->outputPath);
 	buf = uf_markup_buf(buf, "<OutputPath>%s</OutputPath>\n", utf8);
 	g_free(utf8);
-#if 0
-    // Now that outputPath is a GUI entry there is no need to guess it anymore.
-    } else {
-	/* Guess outputPath */
-	char *inPath = g_path_get_dirname(c->inputFilename);
-	char *outPath = g_path_get_dirname(c->outputFilename);
-	if ( strcmp(outPath,".") && strcmp(inPath, outPath) ) {
-	    g_strlcpy(c->outputPath, outPath, max_path);
-	    char *utf8=g_filename_display_name(outPath);
-	    buf = uf_markup_buf(buf, "<OutputPath>%s</OutputPath>\n", utf8);
-	    g_free(utf8);
-	}
-	g_free(outPath);
-	g_free(inPath);
-#endif
     }
 
     /* GUI settings are only saved to .ufrawrc */
@@ -860,6 +882,9 @@ int conf_save(conf_data *c, char *IDFilename, char **confBuffer)
 	if (c->blinkOverUnder!=conf_default.blinkOverUnder)
 	    buf = uf_markup_buf(buf,
 		    "<BlinkOverUnder>%d</BlinkOverUnder>\n", c->blinkOverUnder);
+	if (c->drawLines!=conf_default.drawLines)
+	    buf = uf_markup_buf(buf,
+		    "<DrawLines>%d</DrawLines>\n", c->drawLines);
 	if (c->RememberOutputPath!=conf_default.RememberOutputPath)
 	    buf = uf_markup_buf(buf,
 		    "<RememberOutputPath>%d</RememberOutputPath>\n",
@@ -921,6 +946,12 @@ int conf_save(conf_data *c, char *IDFilename, char **confBuffer)
 	buf = uf_markup_buf(buf,
 		"<WaveletDenoisingThreshold>%d</WaveletDenoisingThreshold>\n",
 		(int)floor(c->threshold));
+#ifdef UFRAW_HOTPIXELS
+    if (c->hotpixel!=conf_default.hotpixel)
+	buf = uf_markup_buf(buf,
+		"<HotpixelSensitivity>%f</HotpixelSensitivity>\n",
+		c->hotpixel);
+#endif
 #ifdef UFRAW_CONTRAST
     if (c->contrast!=conf_default.contrast)
         buf = uf_markup_buf(buf,
@@ -945,6 +976,13 @@ int conf_save(conf_data *c, char *IDFilename, char **confBuffer)
     if (c->saturation!=conf_default.saturation)
 	buf = uf_markup_buf(buf,
 		"<Saturation>%lf</Saturation>\n", c->saturation);
+    for (i = 0; i < max_adjustments; ++i) {
+	lightness_adjustment *a = &c->lightnessAdjustment[i];
+	if (fabs(a->adjustment - conf_default.lightnessAdjustment[i].adjustment) > 0.01) {
+	    buf = uf_markup_buf(buf, "<LightnessAdjustment>%f %f %f</LightnessAdjustment>\n",
+				a->adjustment, a->hue, a->hueWidth);
+	}
+    }
     if (c->grayscaleMode!=conf_default.grayscaleMode)
         buf = uf_markup_buf(buf,
 		"<GrayscaleMode>%s</GrayscaleMode>\n",
@@ -1075,6 +1113,7 @@ int conf_save(conf_data *c, char *IDFilename, char **confBuffer)
 	    if ( j==in_profile && i==0 ) profile = "No";
 	    if ( j==in_profile && i==1 ) profile = "Matrix";
 	    if ( j==out_profile && i==0 ) profile = "sRGB";
+	    if ( j==out_profile && i==1 ) profile = "sRGBEmbedded";
 	    if ( j==display_profile && i==0 ) profile = "System";
 	    if ( j==display_profile && i==1 ) profile = "sRGB";
 	    buf = uf_markup_buf(buf, "<%s%s Current='%s'>%s\n",
@@ -1184,6 +1223,9 @@ void conf_copy_image(conf_data *dst, const conf_data *src)
     g_strlcpy(dst->model, src->model, max_name);
     dst->threshold = src->threshold;
     dst->exposure = src->exposure;
+#ifdef UFRAW_HOTPIXELS
+    dst->hotpixel = src->hotpixel;
+#endif
 #ifdef UFRAW_CONTRAST
     dst->contrast = src->contrast;
 #endif
@@ -1194,6 +1236,9 @@ void conf_copy_image(conf_data *dst, const conf_data *src)
     dst->autoBlack = src->autoBlack;
     dst->restoreDetails = src->restoreDetails;
     dst->clipHighlights = src->clipHighlights;
+    memcpy(dst->lightnessAdjustment, src->lightnessAdjustment,
+	   sizeof dst->lightnessAdjustment);
+    dst->lightnessAdjustmentCount = src->lightnessAdjustmentCount;
     dst->grayscaleMode = src->grayscaleMode;
     memcpy(dst->grayscaleMixer, src->grayscaleMixer,
 	   sizeof dst->grayscaleMixer);
@@ -1285,13 +1330,13 @@ void conf_copy_image(conf_data *dst, const conf_data *src)
     dst->lensdb = src->lensdb;
     if (src->camera)
     {
-        dst->camera = lf_camera_new ();
-        lf_camera_copy (dst->camera, src->camera);
+	dst->camera = lf_camera_new ();
+	lf_camera_copy (dst->camera, src->camera);
     }
     if (src->lens)
     {
-        dst->lens = lf_lens_new ();
-        lf_lens_copy (dst->lens, src->lens);
+	dst->lens = lf_lens_new ();
+	lf_lens_copy (dst->lens, src->lens);
     }
     dst->lens_distortion = src->lens_distortion;
     dst->lens_tca = src->lens_tca;
@@ -1336,6 +1381,7 @@ void conf_copy_save(conf_data *dst, const conf_data *src)
 int conf_set_cmd(conf_data *conf, const conf_data *cmd)
 {
     if (cmd->overwrite!=-1) conf->overwrite = cmd->overwrite;
+    if (cmd->WindowMaximized!=-1) conf->WindowMaximized = cmd->WindowMaximized;
     if (cmd->restoreDetails!=-1)
 	conf->restoreDetails = cmd->restoreDetails;
     if (cmd->clipHighlights!=-1)
@@ -1356,6 +1402,9 @@ int conf_set_cmd(conf_data *conf, const conf_data *cmd)
 	conf->autoExposure = cmd->autoExposure;
     }
     if (cmd->threshold!=NULLF) conf->threshold = cmd->threshold;
+#ifdef UFRAW_HOTPIXELS
+    if (cmd->hotpixel!=NULLF) conf->hotpixel = cmd->hotpixel;
+#endif
 #ifdef UFRAW_CONTRAST
     if (cmd->contrast!=NULLF) conf->contrast = cmd->contrast;
 #endif
@@ -1391,6 +1440,7 @@ int conf_set_cmd(conf_data *conf, const conf_data *cmd)
 		0, cmd->black, 0);
 	conf->autoBlack = disabled_state;
     }
+    if (cmd->smoothing!=-1) conf->smoothing = cmd->smoothing;
     if (cmd->interpolation>=0) conf->interpolation = cmd->interpolation;
     if (cmd->interpolation==obsolete_eahd_interpolation) {
 	conf->interpolation = ahd_interpolation;
@@ -1485,12 +1535,17 @@ N_("--contrast=CONT       Contrast adjustment (default 1.0).\n"),
 N_("--saturation=SAT      Saturation adjustment (default 1.0, 0 for B&W output).\n"),
 N_("--wavelet-denoising-threshold=THRESHOLD\n"
 "                      Wavelet denoising threshold (default 0.0).\n"),
+#ifdef UFRAW_HOTPIXELS
+N_("--hotpixel-sensitivity=VALUE\n"
+"                      Sensitivity for detecting and shaving hot pixels (default 0.0).\n"),
+#endif
 N_("--exposure=auto|EXPOSURE\n"
 "                      Auto exposure or exposure correction in EV (default 0).\n"),
 N_("--black-point=auto|BLACK\n"
 "                      Auto black-point or black-point value (default 0).\n"),
 N_("--interpolation=ahd|vng|four-color|ppg|bilinear\n"
 "                      Interpolation algorithm to use (default ahd).\n"),
+N_("--color-smoothing     Apply color smoothing.\n"),
 N_("--grayscale=none|lightness|luminance|value|mixer\n"
 "                      Grayscale conversion algorithm to use (default none).\n"),
 "\n",
@@ -1504,7 +1559,7 @@ N_("--out-depth=8|16      Output bit depth per channel (default 8).\n"),
 N_("--create-id=no|also|only\n"
 "                      Create no|also|only ID file (default no).\n"),
 N_("--compression=VALUE   JPEG compression (0-100, default 85).\n"),
-N_("--[no]exif            Embed EXIF in output JPEG or PNG (default embed EXIF).\n"),
+N_("--[no]exif            Embed EXIF in output (default embed EXIF).\n"),
 N_("--[no]zip             Enable [disable] TIFF zip compression (default nozip).\n"),
 N_("--embedded-image      Extract the preview image embedded in the raw file\n"
 "                      instead of converting the raw image.\n"),
@@ -1518,10 +1573,11 @@ N_("--out-path=PATH       PATH for output file (default use input file's path).\
 N_("--output=FILE         Output file name, use '-' to output to stdout.\n"),
 N_("--darkframe=FILE      Use FILE for raw darkframe subtraction.\n"),
 N_("--overwrite           Overwrite existing files without asking (default no).\n"),
+N_("--maximize-window     Force window to be maximized.\n"),
 N_("--silent              Do not display any messages during batch conversion.\n"),
 "\n",
 N_("UFRaw first reads the setting from the resource file $HOME/.ufrawrc.\n"
-"Then, if an ID file is specified it setting are read. Next, the setting from\n"
+"Then, if an ID file is specified, its setting are read. Next, the setting from\n"
 "the --conf option are taken, ignoring input/output filenames in the ID file.\n"
 "Lastly, the options from the command line are set. In batch mode, the second\n"
 "group of options is NOT read from the resource file.\n"),
@@ -1596,6 +1652,9 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
 	{ "gamma", 1, 0, 'G'},
 	{ "linearity", 1, 0, 'L'},
 	{ "saturation", 1, 0, 's'},
+#ifdef UFRAW_HOTPIXELS
+	{ "hotpixel-sensitivity", 1, 0, 'H'},
+#endif
 #ifdef UFRAW_CONTRAST
 	{ "contrast", 1, 0, 'y'},
 #endif
@@ -1625,6 +1684,8 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
 	{ "zip", 0, 0, 'z'},
 	{ "nozip", 0, 0, 'Z'},
 	{ "overwrite", 0, 0, 'O'},
+	{ "color-smoothing", 0, 0, 'M' },
+	{ "maximize-window", 0, 0, 'W'},
 	{ "exif", 0, 0, 'E'},
 	{ "noexif", 0, 0, 'F'},
 	{ "embedded-image", 0, 0, 'm'},
@@ -1638,6 +1699,9 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
 	&baseCurveName, &baseCurveFile, &curveName, &curveFile,
 	&cmd->profile[0][0].gamma, &cmd->profile[0][0].linear,
 	&cmd->saturation,
+#ifdef UFRAW_HOTPIXELS
+	&cmd->hotpixel,
+#endif
 #ifdef UFRAW_CONTRAST
 	&cmd->contrast,
 #endif
@@ -1652,12 +1716,16 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
     cmd->autoBlack = disabled_state;
     cmd->losslessCompress=-1;
     cmd->overwrite=-1;
+    cmd->WindowMaximized=-1;
     cmd->embedExif=-1;
     cmd->profile[1][0].BitDepth=-1;
     cmd->embeddedImage=FALSE;
     cmd->silent=FALSE;
     cmd->profile[0][0].gamma=NULLF;
     cmd->profile[0][0].linear=NULLF;
+#ifdef UFRAW_HOTPIXELS
+    cmd->hotpixel=NULLF;
+#endif
 #ifdef UFRAW_CONTRAST
     cmd->contrast=NULLF;
 #endif
@@ -1676,6 +1744,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
     cmd->CropX2 = -1;
     cmd->CropY2 = -1;
     cmd->rotate = -1;
+    cmd->smoothing = -1;
 
     while (1) {
 	c = getopt_long (*argc, *argv, "h", options, &index);
@@ -1696,7 +1765,12 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
 	case 'G':
 	case 'L':
 	case 's':
+#ifdef UFRAW_HOTPIXELS
+	case 'H':
+#endif
+#ifdef UFRAW_CONTRAST
 	case 'y':
+#endif
 	case 'n':
 	    if (sscanf(optarg, "%lf", (double *)optPointer[index])==0) {
 		ufraw_message(UFRAW_ERROR,
@@ -1739,7 +1813,9 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
 	    *(char **)optPointer[index] = optarg;
 	    break;
 	case 'O': cmd->overwrite = TRUE; break;
+	case 'W': cmd->WindowMaximized = TRUE; break;
 	case 'm': cmd->embeddedImage = TRUE; break;
+	case 'M': cmd->smoothing = TRUE; break;
 	case 'q': cmd->silent = TRUE; break;
 	case 'z':
 #ifdef HAVE_LIBZ
@@ -1890,6 +1966,12 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
 	    return -1;
 	}
     }
+    if (cmd->smoothing!=-1) {
+	if ( cmd->interpolation==ahd_interpolation )
+	    cmd->smoothing = 3;
+	else
+	    cmd->smoothing = 1;
+    }
     cmd->grayscaleMode = NULLF;
     if (grayscaleName!=NULL) {
       cmd->grayscaleMode = conf_find_name(grayscaleName, grayscaleModeNames,
