@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_developer.c - functions for developing images or more exactly pixels.
- * Copyright 2004-2008 by Udi Fuchs
+ * Copyright 2004-2009 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@ developer_data *developer_init()
     int i;
     developer_data *d = g_new(developer_data,1);
     d->mode = -1;
+    d->doWB = 1;
     d->gamma = -1;
     d->linear = -1;
     d->saturation = -1;
@@ -60,12 +61,19 @@ developer_data *developer_init()
     TransferFunction[0] = cmsAllocGamma(0x100);
     TransferFunction[1] = TransferFunction[2] = cmsBuildGamma(0x100, 1.0);
     d->saturationProfile = NULL;
+    d->adjustmentProfile = NULL;
     d->intent[out_profile] = -1;
     d->intent[display_profile] = -1;
     d->updateTransform = TRUE;
     d->colorTransform = NULL;
+    d->rgbtolabTransform = NULL;
     d->grayscaleMode = -1;
     d->grayscaleMixer[0] = d->grayscaleMixer[1] = d->grayscaleMixer[2] = -1;
+    for (i=0; i<max_adjustments; i++) { /* Suppress valgrind error. */
+	d->lightnessAdjustment[i].adjustment = 0.0;
+	d->lightnessAdjustment[i].hue = 0.0;
+	d->lightnessAdjustment[i].hueWidth = 0.0;
+    }
     cmsSetErrorHandler(lcms_message);
     return d;
 }
@@ -80,8 +88,11 @@ void developer_destroy(developer_data *d)
     cmsFreeGamma(d->TransferFunction[0]);
     cmsFreeGamma(d->TransferFunction[1]);
     cmsCloseProfile(d->saturationProfile);
+    cmsCloseProfile(d->adjustmentProfile);
     if (d->colorTransform!=NULL)
 	cmsDeleteTransform(d->colorTransform);
+    if (d->rgbtolabTransform!=NULL)
+	cmsDeleteTransform(d->rgbtolabTransform);
     g_free(d);
 }
 
@@ -232,6 +243,73 @@ static cmsHPROFILE create_contrast_saturation_profile(double contrast,
     return hICC;
 }
 
+static int luminance_adjustment_sampler(WORD In[], WORD Out[], LPVOID Cargo)
+{
+    cmsCIELab Lab;
+    cmsCIELCh LCh;
+    const developer_data *d = Cargo;
+    const lightness_adjustment *a;
+
+    cmsLabEncoded2Float(&Lab, In);
+    cmsLab2LCh(&LCh, &Lab);
+
+    double adj = 0.0;
+    int i;
+    for (i=0, a=d->lightnessAdjustment; i<max_adjustments; i++, a++) {
+	double deltaHue = fabs(LCh.h - a->hue);
+	double hueWidth = MAX(a->hueWidth, 360.0/33.0);
+	if (deltaHue > 180.0)
+	    deltaHue = 360.0 - deltaHue;
+	if (deltaHue > hueWidth)
+	    continue;
+	/* This assigns the scales on a nice curve. */
+	double scale = cos(deltaHue / hueWidth * (M_PI / 2) );
+	adj += (a->adjustment - 1) * (scale * scale);
+    }
+    /* The adjustment is scaled based on the colorfulness of the point,
+     * since uncolored pixels should not be adjusted.  However, few
+     * (s)RGB colors have a colorfulness value larger than 1/2 of
+     * max_colorfulness, so use that as an actual maximum colorfulness. */
+    adj = adj * MIN(LCh.C / (max_colorfulness / 2), 1.0) + 1;
+    LCh.L *= adj;
+
+    cmsLCh2Lab(&Lab, &LCh);
+    cmsFloat2LabEncoded(Out, &Lab);
+
+    return TRUE;
+}
+
+/* Based on lcms' cmsCreateBCHSWabstractProfile() */
+static cmsHPROFILE create_adjustment_profile(const developer_data *d)
+{
+    cmsHPROFILE hICC;
+    LPLUT Lut;
+
+    hICC = _cmsCreateProfilePlaceholder();
+    if (hICC==NULL) return NULL;// can't allocate
+
+    cmsSetDeviceClass(hICC, icSigAbstractClass);
+    cmsSetColorSpace(hICC, icSigLabData);
+    cmsSetPCS(hICC, icSigLabData);
+    cmsSetRenderingIntent(hICC, INTENT_PERCEPTUAL);
+
+    // Creates a LUT with 3D grid only
+    Lut = cmsAllocLUT();
+    cmsAlloc3DGrid(Lut, 33, 3, 3);
+    if (!cmsSample3DGrid(Lut, luminance_adjustment_sampler, (void*)d, 0)) {
+	// Shouldn't reach here
+	cmsFreeLUT(Lut);
+	cmsCloseProfile(hICC);
+	return NULL;
+    }
+    // Create tags
+    cmsAddTag(hICC, icSigMediaWhitePointTag, (LPVOID) cmsD50_XYZ());
+    cmsAddTag(hICC, icSigAToB0Tag, (LPVOID) Lut);
+    // LUT is already on virtual profile
+    cmsFreeLUT(Lut);
+    return hICC;
+}
+
 /* Find a for which (1-exp(-a x)/(1-exp(-a)) has derivative b at x=0 */
 /* In other words, solve a/(1-exp(-a))==b */
 static double findExpCoeff(double b)
@@ -256,6 +334,8 @@ static void developer_create_transform(developer_data *d, DeveloperMode mode)
     d->updateTransform = FALSE;
     if (d->colorTransform!=NULL)
 	cmsDeleteTransform(d->colorTransform);
+    if (d->rgbtolabTransform!=NULL)
+	cmsDeleteTransform(d->rgbtolabTransform);
 
     int targetProfile;
     if ( mode==file_developer || mode==auto_developer ) {
@@ -269,22 +349,28 @@ static void developer_create_transform(developer_data *d, DeveloperMode mode)
 	/* No need for proofing transformation. */
 	if ( strcmp(d->profileFile[in_profile],"")==0 &&
 	     strcmp(d->profileFile[targetProfile],"")==0 &&
-	     d->luminosityProfile==NULL && d->saturationProfile==NULL ) {
+	     d->luminosityProfile==NULL &&
+	     d->adjustmentProfile==NULL &&
+	     d->saturationProfile==NULL ) {
 	    /* No transformation at all. */
 	    d->colorTransform = NULL;
 #if defined(LCMS_VERSION) && LCMS_VERSION <= 113 /* Bypass a lcms 1.13 bug. */
-	} else if ( d->luminosityProfile==NULL && d->saturationProfile==NULL ) {
+	} else if ( d->luminosityProfile==NULL
+		    && d->adjustmentProfile==NULL
+		    && d->saturationProfile==NULL ) {
 	    d->colorTransform = cmsCreateTransform(
 		    d->profile[in_profile], TYPE_RGB_16,
 		    d->profile[targetProfile], TYPE_RGB_16,
 		    d->intent[out_profile], 0);
 #endif
 	} else {
-	    cmsHPROFILE prof[4];
+	    cmsHPROFILE prof[5];
 	    int i = 0;
 	    prof[i++] = d->profile[in_profile];
 	    if ( d->luminosityProfile!=NULL )
 		prof[i++] = d->luminosityProfile;
+	    if ( d->adjustmentProfile!=NULL )
+		prof[i++] = d->adjustmentProfile;
 	    if ( d->saturationProfile!=NULL )
 		prof[i++] = d->saturationProfile;
 	    prof[i++] = d->profile[targetProfile];
@@ -293,7 +379,9 @@ static void developer_create_transform(developer_data *d, DeveloperMode mode)
 	}
     } else {
 	/* Create a proofing profile */
-	if ( d->luminosityProfile==NULL && d->saturationProfile==NULL ) {
+	if ( d->luminosityProfile==NULL
+	     && d->adjustmentProfile==NULL
+	     && d->saturationProfile==NULL ) {
 	    /* No intermediate profiles, we can use lcms proofing directly. */
 	    d->colorTransform = cmsCreateProofingTransform(
 		    d->profile[in_profile], TYPE_RGB_16,
@@ -305,11 +393,13 @@ static void developer_create_transform(developer_data *d, DeveloperMode mode)
 	    /* Following code imitates the function
 	     * cmsCreateMultiprofileProofingTransform(),
 	     * which does not exist in lcms. */
-	    cmsHPROFILE prof[3];
+	    cmsHPROFILE prof[4];
 	    int i = 0;
 	    prof[i++] = d->profile[in_profile];
 	    if ( d->luminosityProfile!=NULL )
 		prof[i++] = d->luminosityProfile;
+	    if ( d->adjustmentProfile!=NULL )
+		prof[i++] = d->adjustmentProfile;
 	    if ( d->saturationProfile!=NULL )
 		prof[i++] = d->saturationProfile;
 	    d->colorTransform = cmsCreateMultiprofileTransform(prof, i,
@@ -327,6 +417,21 @@ static void developer_create_transform(developer_data *d, DeveloperMode mode)
 		    cmsFLAGS_SOFTPROOFING);
 	}
     }
+    d->rgbtolabTransform = cmsCreateTransform(d->profile[in_profile], 
+					      TYPE_RGB_16,
+					      cmsCreateLabProfile(cmsD50_xyY()),
+					      TYPE_Lab_16,
+					      INTENT_ABSOLUTE_COLORIMETRIC, 0);
+}
+
+static gboolean test_adjustments(const lightness_adjustment values[max_adjustments],
+				 gdouble reference, gdouble threshold)
+{
+    int i;
+    for (i = 0; i < max_adjustments; ++i)
+	if (fabs(values[i].adjustment - reference) >= threshold)
+	    return TRUE;
+    return FALSE;
 }
 
 void developer_prepare(developer_data *d, conf_data *conf,
@@ -495,6 +600,18 @@ void developer_prepare(developer_data *d, conf_data *conf,
 	}
 	d->updateTransform = TRUE;
     }
+    if (memcmp(d->lightnessAdjustment, conf->lightnessAdjustment,
+	       sizeof d->lightnessAdjustment) != 0) {
+	/* Adjustments have changed, need to update them. */
+	d->updateTransform = TRUE;
+	memcpy(d->lightnessAdjustment, conf->lightnessAdjustment,
+	       sizeof d->lightnessAdjustment);
+	cmsCloseProfile(d->adjustmentProfile);
+	d->adjustmentProfile = test_adjustments(d->lightnessAdjustment, 1.0, 0.01)
+	    ? create_adjustment_profile(d)
+	    : NULL;
+    }
+
     if ( conf->saturation!=d->saturation
 #ifdef UFRAW_CONTRAST
 	 || conf->contrast!=d->contrast
@@ -525,13 +642,39 @@ void developer_prepare(developer_data *d, conf_data *conf,
     developer_create_transform(d, mode);
 }
 
+static void apply_matrix(const developer_data *d,
+			 const gint64 in[4],
+			 gint64 out[3])
+{
+    gint64 tmp[3];
+    unsigned c, cc;
+    for (cc=0; cc<3; cc++) {
+	tmp[cc] = 0;
+	for (c=0; c<d->colors; c++)
+	    tmp[cc] += in[c] * d->colorMatrix[cc][c];
+    }
+    for (cc=0; cc<3; cc++)
+	out[cc] = MAX(tmp[cc]/0x10000, 0);
+}
+
+static void cond_apply_matrix(const developer_data *d,
+			      const gint64 in[4],
+			      gint64 out[3])
+{
+    if (d->useMatrix)
+	apply_matrix(d, in, out);
+    else
+	memcpy(out, in, 3 * sizeof out[0]);
+}
+
 extern const double xyz_rgb[3][3];
 static const double rgb_xyz[3][3] = {			/* RGB from XYZ */
     { 3.24048, -1.53715, -0.498536 },
     { -0.969255, 1.87599, 0.0415559 },
     { 0.0556466, -0.204041, 1.05731 } };
 
-static void rgb_to_cielch(gint64 rgb[3], float lch[3])
+// Convert linear RGB to CIE-LCh
+void uf_rgb_to_cielch(gint64 rgb[3], float lch[3])
 {
     int c, cc, i;
     float r, xyz[3], lab[3];
@@ -562,7 +705,8 @@ static void rgb_to_cielch(gint64 rgb[3], float lch[3])
     lch[2] = atan2(lab[2], lab[1]);
 }
 
-static void cielch_to_rgb(float lch[3], gint64 rgb[3])
+// Convert CIE-LCh to linear RGB
+void uf_cielch_to_rgb(float lch[3], gint64 rgb[3])
 {
     int c, cc;
     float xyz[3], fx, fy, fz, xr, yr, zr, kappa, epsilon, tmpf, lab[3];
@@ -590,29 +734,75 @@ static void cielch_to_rgb(float lch[3], gint64 rgb[3])
     }
 }
 
-static void MaxMidMin(gint64 p[3], int *maxc, int *midc, int *minc)
+void uf_raw_to_cielch(const developer_data *d,
+		      const guint16 raw[4],
+		      float lch[3])
 {
-    if (p[0] > p[1] && p[0] > p[2]) {
-	*maxc = 0;
-	if (p[1] > p[2]) { *midc = 1; *minc = 2; }
-	else { *midc = 2; *minc = 1; }
-    } else if (p[1] > p[2]) {
-	*maxc = 1;
-	if (p[0] > p[2]) { *midc = 0; *minc = 2; }
-	else { *midc = 2; *minc = 0; }
-    } else {
-	*maxc = 2;
-	if (p[0] > p[1]) { *midc = 0; *minc = 1; }
-	else { *midc = 1; *minc = 0; }
+    gint64 tmp[4];
+    guint16 rgbpixel[3];
+    guint16 labpixel[3];
+    cmsCIELab Lab;
+    cmsCIELCh LCh;
+    unsigned int c;
+
+    for (c = 0; c < d->colors; ++c) {
+	tmp[c] = raw[c];
+	if (d->doWB) {
+	    tmp[c] *= d->rgbWB[c];
+	    tmp[c] /= 0x10000;
+	}
     }
+    cond_apply_matrix(d, tmp, tmp);
+    for (c = 0; c < 3; ++c)
+	rgbpixel[c] = tmp[c];
+
+    cmsDoTransform(d->rgbtolabTransform, rgbpixel, labpixel, 1);
+
+    cmsLabEncoded2Float(&Lab, labpixel);
+    cmsLab2LCh(&LCh, &Lab);
+    lch[0] = LCh.L;
+    lch[1] = LCh.C;
+    lch[2] = LCh.h;
 }
 
-inline void develope(void *po, guint16 pix[4], developer_data *d, int mode,
-    guint16 *buf, int count)
+static void MaxMidMin(const gint64 p[3], int *maxc, int *midc, int *minc)
 {
-    guint8 *p8 = po;
-    guint16 *p16 = po, c, tmppix[3];
+    gint64 a = p[0];
+    gint64 b = p[1];
+    gint64 c = p[2];
+    int max = 0;
+    int mid = 1;
+    int min = 2;
+
+    if (a < b) {
+	gint64 tmp = b;
+	b = a;
+	a = tmp;
+	max = 1;
+	mid = 0;
+    }
+    if (b < c) {
+	b = c;
+	min = mid;
+	mid = 2;
+	if (a < b) {
+	    int tmp = max;
+	    max = mid;
+	    mid = tmp;
+	}
+    }
+
+    *maxc = max;
+    *midc = mid;
+    *minc = min;
+}
+
+void develop(void *po, guint16 pix[4], developer_data *d, int mode, int count)
+{
+    guint16 c, tmppix[3], *buf;
     int i;
+    if (mode==16) buf = po;
+    else buf = g_alloca(count*6);
 
 #ifdef _OPENMP
 #pragma omp parallel				\
@@ -643,8 +833,10 @@ inline void develope(void *po, guint16 pix[4], developer_data *d, int mode,
 	cmsDoTransform(d->colorTransform, buf, buf, count);
 #endif
 
-    if (mode==16) for (i=0; i<3*count; i++) p16[i] = buf[i];
-    else for (i=0; i<3*count; i++) p8[i] = buf[i] >> 8;
+    if (mode!=16) {
+	guint8 *p8 = po;
+	for (i=0; i<3*count; i++) p8[i] = buf[i] >> 8;
+    }
 }
 
 static void develop_grayscale(guint16 *pixel, const developer_data *d)
@@ -687,12 +879,16 @@ static void develop_grayscale(guint16 *pixel, const developer_data *d)
 
 void develop_linear(guint16 in[4], guint16 out[3], developer_data *d)
 {
-    unsigned c, cc;
-    gint64 tmppix[4], tmp;
+    unsigned c;
+    gint64 tmppix[4];
     gboolean clipped = FALSE;
     for (c=0; c<d->colors; c++) {
 	/* Set WB, normalizing tmppix[c]<0x10000 */
-	tmppix[c] = (guint64)in[c] * d->rgbWB[c] / 0x10000;
+	tmppix[c] = in[c];
+	if (d->doWB) {
+	    tmppix[c] *= d->rgbWB[c];
+	    tmppix[c] /= 0x10000;
+	}
 	if ( d->restoreDetails!=clip_details &&
 	    tmppix[c] > d->max ) {
 	    clipped = TRUE;
@@ -711,34 +907,18 @@ void develop_linear(guint16 in[4], guint16 out[3], developer_data *d)
 	 * to "1.0" (full exposure). Still the maximal value can be
 	 * d->exposure * 0x10000 / d->max */
 	gint64 unclippedPix[3], clippedPix[3];
-	if ( d->useMatrix ) {
-	    for (cc=0; cc<3; cc++) {
-		for (c=0, tmp=0; c<d->colors; c++)
-		    tmp += tmppix[c] * d->colorMatrix[cc][c];
-		unclippedPix[cc] = MAX(tmp/0x10000, 0);
-	    }
-	} else {
-	    for (c=0; c<3; c++) unclippedPix[c] = tmppix[c];
-	}
+	cond_apply_matrix(d, tmppix, unclippedPix);
 	for (c=0; c<3; c++) tmppix[c] = MIN(tmppix[c], d->exposure);
-	if ( d->useMatrix ) {
-	    for (cc=0; cc<3; cc++) {
-		for (c=0, tmp=0; c<d->colors; c++)
-		    tmp += tmppix[c] * d->colorMatrix[cc][c];
-		clippedPix[cc] = MAX(tmp/0x10000, 0);
-	    }
-	} else {
-	    for (c=0; c<3; c++) clippedPix[c] = tmppix[c];
-	}
+	cond_apply_matrix(d, tmppix, clippedPix);
 	if ( d->restoreDetails==restore_lch_details ) {
 	    float lch[3], clippedLch[3], unclippedLch[3];
-	    rgb_to_cielch(unclippedPix, unclippedLch);
-	    rgb_to_cielch(clippedPix, clippedLch);
+	    uf_rgb_to_cielch(unclippedPix, unclippedLch);
+	    uf_rgb_to_cielch(clippedPix, clippedLch);
 	    //lch[0] = clippedLch[0] + (unclippedLch[0]-clippedLch[0]) * x;
 	    lch[0] = unclippedLch[0];
 	    lch[1] = clippedLch[1];
 	    lch[2] = clippedLch[2];
-	    cielch_to_rgb(lch, tmppix);
+	    uf_cielch_to_rgb(lch, tmppix);
 	} else { /* restore_hsv_details */
 	    int maxc, midc, minc;
 	    MaxMidMin(unclippedPix, &maxc, &midc, &minc);
@@ -784,14 +964,8 @@ void develop_linear(guint16 in[4], guint16 out[3], developer_data *d)
 	    tmppix[midc] = lum * (0x10000-sat + sat*hue/0x10000) / 0x10000;
 	}
     } else { /* !clipped */
-	if (d->useMatrix) {
-	    gint64 tmp[3];
-	    for (cc=0; cc<3; cc++) {
-		for (c=0, tmp[cc]=0; c<d->colors; c++)
-		    tmp[cc] += tmppix[c] * d->colorMatrix[cc][c];
-	    }
-	    for (c=0; c<3; c++) tmppix[c] = MAX(tmp[c]/0x10000, 0);
-	}
+	if (d->useMatrix)
+	    apply_matrix(d, tmppix, tmppix);
 	gint64 max = tmppix[0];
 	for (c=1; c<3; c++) max = MAX(tmppix[c], max);
 	if (max > 0xFFFF) {
