@@ -151,6 +151,12 @@ try {
 	    != exifData.end() ) {
 	uf_strlcpy_to_utf8(uf->conf->focalLenText, max_name, pos, exifData);
 	uf->conf->focal_len = pos->toFloat ();
+#if EXIV2_TEST_VERSION(0,18,1)
+    } else if ( (pos=exifData.findKey(Exiv2::ExifKey(
+			"Exif.Canon.FocalLength"))) != exifData.end() ) {
+	uf_strlcpy_to_utf8(uf->conf->focalLenText, max_name, pos, exifData);
+	uf->conf->focal_len = pos->toFloat ();
+#endif
     }
     /* Read focal length in 35mm equivalent */
     if ( (pos=exifData.findKey(Exiv2::ExifKey(
@@ -158,10 +164,20 @@ try {
 	    != exifData.end() ) {
 	uf_strlcpy_to_utf8(uf->conf->focalLen35Text, max_name, pos, exifData);
     }
-    /* Read lens name */
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Nikon3.Lens")))
+    /* Read full lens name */
+    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Nikon3.LensData")))
 	    != exifData.end() ) {
 	uf_strlcpy_to_utf8(uf->conf->lensText, max_name, pos, exifData);
+	/* Read short lens name if full lens name if not defined */
+	if ( isdigit(uf->conf->lensText[0]) ) {
+	    ufraw_message(UFRAW_SET_LOG,
+		    "Value of EXIF tag 'Exif.Nikon3.LensData' is not defined. "
+		    "Reading 'Exif.Nikon3.Lens'.\n");
+	    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Nikon3.Lens")))
+		    != exifData.end() ) {
+		uf_strlcpy_to_utf8(uf->conf->lensText, max_name, pos, exifData);
+	    }
+	}
 #if EXIV2_TEST_VERSION(0,17,91)		/* Exiv2 0.18-pre1 */
     } else if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.CanonCs.LensType")))
 		!= exifData.end() ) {
@@ -307,6 +323,11 @@ static Exiv2::ExifData ufraw_prepare_exifdata(ufraw_data *uf)
     if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.NikonPreview.JPEGInterchangeFormat")))
 	    != exifData.end() )
 	exifData.erase(pos);
+
+    // DNG private data
+    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Image.DNGPrivateData")))
+	    != exifData.end() )
+	exifData.erase(pos);
 #endif
 
 #if EXIV2_TEST_VERSION(0,16,0)
