@@ -3,7 +3,7 @@
  *
  * ufraw_lens_ui.c - User interface for interaction with lensfun,
  *                   a lens defect correction library.
- * Copyright 2007-2008 by Andrew Zabolotny
+ * Copyright 2007-2009 by Andrew Zabolotny
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -420,7 +420,8 @@ static void lens_set (preview_data *data, const lfLens *lens)
         return;
     }
 
-    lf_lens_copy (CFG->lens, lens);
+    if (CFG->lens!= lens)
+	lf_lens_copy (CFG->lens, lens);
 
     maker = lf_mlstr_get (lens->Maker);
     model = lf_mlstr_get (lens->Model);
@@ -655,15 +656,13 @@ static GtkAdjustment *append_term (
 
     GtkAdjustment *adj = adjustment_scale (
         GTK_TABLE (table), 0, y, param->Name, *term, term,
-        param->Min, param->Max, step, page, accuracy, NULL, callback);
+        param->Min, param->Max, step, page, accuracy, FALSE, NULL, callback,
+	NULL, NULL, NULL);
 
-    GtkWidget *button = gtk_button_new ();
-    gtk_container_add (GTK_CONTAINER (button),
-                       gtk_image_new_from_stock (GTK_STOCK_REFRESH, GTK_ICON_SIZE_BUTTON));
+    GtkWidget *button = stock_icon_button(GTK_STOCK_REFRESH, NULL,
+                      G_CALLBACK (reset_adjustment_value), (void *)param);
     gtk_table_attach (GTK_TABLE (table), button, 7, 8, y, y + 1, 0, 0, 0, 0);
     g_object_set_data (G_OBJECT(button), "Adjustment", adj);
-    g_signal_connect (G_OBJECT (button), "clicked",
-                      G_CALLBACK (reset_adjustment_value), (void *)param);
 
     return adj;
 }
@@ -789,7 +788,7 @@ static void fill_tca_page (preview_data *data)
     uf_widget_set_tooltip(data->LensTCAModel,
                           _("Chromatic Aberrations mathematical model"));
     active_index = 0;
-    for (i = 0; ; i++)
+    for (i = 0; i <= LF_TCA_MODEL_LINEAR; i++)
     {
         const char *model_name;
         lfTCAModel model = LF_TCA_MODEL_NONE + i;
@@ -906,7 +905,7 @@ static void fill_vignetting_page (preview_data *data)
     uf_widget_set_tooltip(data->LensVignettingModel,
                           _("Optical vignetting mathematical model"));
     active_index = 0;
-    for (i = 0; ; i++)
+    for (i = 0; i <= LF_VIGNETTING_MODEL_PA; i++)
     {
         const char *model_name;
         lfVignettingModel model = LF_VIGNETTING_MODEL_NONE + i;
@@ -999,7 +998,10 @@ static void distortion_model_changed (GtkComboBox *widget, preview_data *data)
     gtk_label_set_text (GTK_LABEL (data->LensDistortionDesc), details);
     gtk_widget_show_all (data->LensDistortionTable);
 
-    preview_invalidate_layer (data, ufraw_lensfun_phase);
+    if (CFG->lens_distortion.Model == LF_DIST_MODEL_NONE)
+	preview_invalidate_layer (data, ufraw_develop_phase);
+    else
+	preview_invalidate_layer (data, ufraw_lensfun_phase);
     render_preview (data);
 }
 
@@ -1023,7 +1025,7 @@ static void fill_distortion_page (preview_data *data)
     uf_widget_set_tooltip (data->LensDistortionModel,
                            _("Lens distortion mathematical model"));
     active_index = 0;
-    for (i = 0; ; i++)
+    for (i = 0; i <= LF_DIST_MODEL_PTLENS; i++)
     {
         const char *model_name;
         lfDistortionModel model = LF_DIST_MODEL_NONE + i;
@@ -1131,7 +1133,7 @@ static void fill_geometry_page (preview_data *data)
     gtk_table_attach (GTK_TABLE (data->LensGeometryTable), data->LensToGeometryDesc,
                       0, 2, 3, 4, GTK_EXPAND | GTK_FILL, 0, 0, 10);
 
-    for (i = 0; ; i++)
+    for (i = 0; i <= LF_EQUIRECTANGULAR; i++)
     {
         lfLensType type = LF_UNKNOWN + i;
         const char *type_name = lf_get_lens_type_desc (type, NULL);
@@ -1171,23 +1173,16 @@ void lens_fill_interface (preview_data *data, GtkWidget *page)
     gtk_table_attach(table, data->CameraModel, 1, 2, 0, 1,
                      GTK_EXPAND|GTK_FILL, 0, 2, 0);
 
-    button = gtk_button_new();
-    gtk_container_add(GTK_CONTAINER(button),
-                      gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_BUTTON));
-    uf_widget_set_tooltip(button,
+    button = stock_icon_button(GTK_STOCK_FIND,
                           _("Search for camera using a pattern\n"
-                            "Format: [Maker, ][Model]"));
-    gtk_table_attach(table, button, 2, 3, 0, 1, 0, 0, 0, 0);
-    g_signal_connect(G_OBJECT(button), "clicked",
+                            "Format: [Maker, ][Model]"),
                      G_CALLBACK(camera_search_clicked), data);
+    gtk_table_attach(table, button, 2, 3, 0, 1, 0, 0, 0, 0);
 
-    button = gtk_button_new();
-    gtk_container_add(GTK_CONTAINER(button),
-                      gtk_image_new_from_stock(GTK_STOCK_INDEX, GTK_ICON_SIZE_BUTTON));
-    uf_widget_set_tooltip(button, _("Choose camera from complete list"));
-    gtk_table_attach(table, button, 3, 4, 0, 1, 0, 0, 0, 0);
-    g_signal_connect(G_OBJECT(button), "clicked",
+    button = stock_icon_button(GTK_STOCK_INDEX,
+		    _("Choose camera from complete list"),
                      G_CALLBACK(camera_list_clicked), data);
+    gtk_table_attach(table, button, 3, 4, 0, 1, 0, 0, 0, 0);
 
     /* Lens selector */
     label = gtk_label_new(_("Lens"));
@@ -1199,23 +1194,16 @@ void lens_fill_interface (preview_data *data, GtkWidget *page)
     gtk_table_attach(table, data->LensModel, 1, 2, 1, 2,
                      GTK_EXPAND|GTK_FILL, 0, 2, 0);
 
-    button = gtk_button_new();
-    gtk_container_add(GTK_CONTAINER(button),
-                      gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_BUTTON));
-    uf_widget_set_tooltip(button,
+    button = stock_icon_button(GTK_STOCK_FIND,
                           _("Search for lens using a pattern\n"
-                            "Format: [Maker, ][Model]"));
-    gtk_table_attach(table, button, 2, 3, 1, 2, 0, 0, 0, 0);
-    g_signal_connect(G_OBJECT(button), "clicked",
+                            "Format: [Maker, ][Model]"),
                      G_CALLBACK(lens_search_clicked), data);
+    gtk_table_attach(table, button, 2, 3, 1, 2, 0, 0, 0, 0);
 
-    button = gtk_button_new();
-    gtk_container_add(GTK_CONTAINER(button),
-                      gtk_image_new_from_stock(GTK_STOCK_INDEX, GTK_ICON_SIZE_BUTTON));
-    uf_widget_set_tooltip(button, _("Choose lens from list of possible variants"));
-    gtk_table_attach(table, button, 3, 4, 1, 2, 0, 0, 0, 0);
-    g_signal_connect(G_OBJECT(button), "clicked",
+    button = stock_icon_button(GTK_STOCK_INDEX,
+		     _("Choose lens from list of possible variants"),
                      G_CALLBACK(lens_list_clicked), data);
+    gtk_table_attach(table, button, 3, 4, 1, 2, 0, 0, 0, 0);
 
     data->LensParamBox = gtk_hbox_new (FALSE, 0);
     gtk_box_pack_start (GTK_BOX (page), data->LensParamBox, FALSE, FALSE, 2);
@@ -1225,26 +1213,18 @@ void lens_fill_interface (preview_data *data, GtkWidget *page)
 
     data->LensScaleAdjustment = adjustment_scale (
         subTable, 0, 0, _("Scale"), CFG->lens_scale, &CFG->lens_scale,
-        -3, 3, 0.001, 0.1, 3, _("Image scale power-of-two"),
-        G_CALLBACK (lens_scale_update));
+        -3, 3, 0.001, 0.1, 3, FALSE, _("Image scale power-of-two"),
+        G_CALLBACK (lens_scale_update), NULL, NULL, NULL);
 
-    data->LensScaleResetButton = gtk_button_new ();
-    gtk_container_add (GTK_CONTAINER (data->LensScaleResetButton),
-        gtk_image_new_from_stock (GTK_STOCK_REFRESH, GTK_ICON_SIZE_BUTTON));
+    data->LensScaleResetButton = stock_icon_button(
+	GTK_STOCK_REFRESH, _("Reset image scale to default"),
+	G_CALLBACK (lens_scale_reset), NULL);
     gtk_table_attach (subTable, data->LensScaleResetButton, 7, 8, 0, 1, 0,0,0,0);
-    uf_widget_set_tooltip (data->LensScaleResetButton,
-                           _("Reset image scale to default"));
-    g_signal_connect(G_OBJECT(data->LensScaleResetButton), "clicked",
-                     G_CALLBACK (lens_scale_reset), NULL);
 
-    data->LensAutoScaleButton = gtk_button_new ();
-    gtk_container_add (GTK_CONTAINER (data->LensAutoScaleButton),
-        gtk_image_new_from_stock (GTK_STOCK_ZOOM_FIT, GTK_ICON_SIZE_BUTTON));
+    data->LensAutoScaleButton = stock_icon_button(
+        GTK_STOCK_ZOOM_FIT, _("Autoscale the image for best fit"),
+	G_CALLBACK (lens_autoscale), NULL);
     gtk_table_attach (subTable, data->LensAutoScaleButton, 8, 9, 0, 1, 0,0,0,0);
-    uf_widget_set_tooltip (data->LensAutoScaleButton,
-                           _("Autoscale the image for best fit"));
-    g_signal_connect(G_OBJECT(data->LensAutoScaleButton), "clicked",
-                     G_CALLBACK (lens_autoscale), NULL);
 
     GtkNotebook *subnb = GTK_NOTEBOOK(gtk_notebook_new());
     gtk_box_pack_start(GTK_BOX(page), GTK_WIDGET(subnb),
@@ -1284,7 +1264,15 @@ void lens_fill_interface (preview_data *data, GtkWidget *page)
     fill_distortion_page (data);
     fill_geometry_page (data);
 
-    lens_set (data, CFG->lens);
+    const lfLens **lenses = NULL;
+    if (strlen(CFG->lensText) > 0)
+	lenses = lf_db_find_lenses_hd(CFG->lensdb,
+		CFG->camera, NULL, CFG->lensText, 0);
+    if (lenses!=NULL) {
+	lf_lens_copy(CFG->lens, lenses[0]);
+	lf_free(lenses);
+    }
+    lens_set(data, CFG->lens);
 }
 
 #endif /* HAVE_LENSFUN */
