@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_ufraw.c - program interface to all the components
- * Copyright 2004-2008 by Udi Fuchs
+ * Copyright 2004-2009 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -159,7 +159,6 @@ ufraw_data *ufraw_open(char *filename)
 	    g_free(conf);
 	    return NULL;
 	}
-	if (conf->createID==only_id) conf->createID = also_id;
 
 	/* If inputFilename and outputFilename have the same path,
 	 * then inputFilename is searched for in the path of the ID file.
@@ -220,7 +219,6 @@ ufraw_data *ufraw_open(char *filename)
     uf->unzippedBufLen = unzippedBufLen;
     uf->conf = conf;
     g_strlcpy(uf->filename, filename, max_path);
-    uf->image.image = NULL;
     int i;
     for (i=ufraw_first_phase; i<ufraw_phases_num; i++)
 	uf->Images[i].buffer = NULL;
@@ -313,21 +311,17 @@ int ufraw_load_darkframe(ufraw_data *uf)
     return UFRAW_SUCCESS;
 }
 
-void ufraw_update_rotated_dimensions(ufraw_data *uf)
-{
-    double rotationRadians = (uf->conf->rotationAngle * 2 * M_PI) / 360;
-    uf->rotatedWidth = ceil((uf->initialHeight * sin(rotationRadians))
-	+ (uf->initialWidth * cos(rotationRadians)));
-    uf->rotatedHeight = ceil((uf->initialWidth * sin(rotationRadians))
-	+ (uf->initialHeight * cos(rotationRadians)));
-}
-
 void ufraw_get_image_dimensions(dcraw_data *raw, ufraw_data *uf)
 {
     dcraw_image_dimensions(raw, uf->conf->orientation,
 	    &uf->initialHeight, &uf->initialWidth);
 
-    ufraw_update_rotated_dimensions(uf);
+    // update rotated dimensions
+    double rotationRadians = (uf->conf->rotationAngle * 2 * M_PI) / 360;
+    uf->rotatedWidth = ceil((uf->initialHeight * sin(rotationRadians))
+	+ (uf->initialWidth * cos(rotationRadians)));
+    uf->rotatedHeight = ceil((uf->initialWidth * sin(rotationRadians))
+	+ (uf->initialHeight * cos(rotationRadians)));
 
     if (uf->conf->CropX1 < 0) uf->conf->CropX1 = 0;
     if (uf->conf->CropY1 < 0) uf->conf->CropY1 = 0;
@@ -428,9 +422,9 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
 #ifdef __MINGW32__
     /* MinG32 does not have ctime_r(). */
     g_strlcpy(uf->conf->timestampText, ctime(&raw->timestamp), max_name);
-#elif defined(__sun) /* Solaris */
+#elif defined(__sun) && !defined(_POSIX_PTHREAD_SEMANTICS) /* Solaris */
     /*
-     * Some (all?) versions of Solaris followed a draft POSIX.1c standard
+     * Some versions of Solaris followed a draft POSIX.1c standard
      * where ctime_r took a third length argument.
      */
     ctime_r(&raw->timestamp, uf->conf->timestampText,
@@ -446,24 +440,16 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
 
     uf->conf->timestamp = raw->timestamp;
 
+    uf->conf->CameraOrientation = raw->flip;
+
     if ( !uf->conf->rotate ) {
-	uf->conf->orientation = 1;
+	uf->conf->orientation = 0;
 	uf->conf->rotationAngle = 0;
     } else {
 	if ( !uf->LoadingID || uf->conf->orientation<0 )
-	    uf->conf->orientation = raw->flip;
-
+	    uf->conf->orientation = uf->conf->CameraOrientation;
 	// Normalise rotations to a flip, then rotation of 0 < a < 90 degrees.
-	uf->conf->rotationAngle = fmod(uf->conf->rotationAngle, 360.0);
-	int angle, flip = 0;
-	angle = floor(uf->conf->rotationAngle/90)*90;
-	switch (angle) {
-	    case  90: flip = 6; break;
-	    case 180: flip = 3; break;
-	    case 270: flip = 5; break;
-	}
-	ufraw_flip_orientation(uf, flip);
-	uf->conf->rotationAngle -= angle;
+	ufraw_normalize_rotation(uf);
     }
 
     if (uf->inputExifBuf==NULL) {
@@ -635,9 +621,8 @@ void ufraw_close(ufraw_data *uf)
 //    g_free(uf->conf);
     g_free(uf->inputExifBuf);
     g_free(uf->outputExifBuf);
-    g_free(uf->image.image);
     int i;
-    for (i=ufraw_first_phase+1; i<ufraw_phases_num; i++)
+    for (i=ufraw_first_phase; i<ufraw_phases_num; i++)
 	g_free(uf->Images[i].buffer);
     g_free(uf->thumb.buffer);
     developer_destroy(uf->developer);
@@ -697,34 +682,23 @@ void ufraw_developer_prepare(ufraw_data *uf, DeveloperMode mode)
 int ufraw_convert_image_init(ufraw_data *uf)
 {
     dcraw_data *raw = uf->raw;
-    int temp_height, temp_width;
-    // This code is copied from dcraw_image_dimensions().
-    if (raw->pixel_aspect < 1)
-	temp_height = (int)(raw->height / raw->pixel_aspect + 0.5);
-    else
-	temp_height = raw->height;
-    if (raw->pixel_aspect > 1)
-	temp_width = (int)(raw->width * raw->pixel_aspect + 0.5);
-    else
-	temp_width = raw->width;
 
+    uf->ConvertShrink = 1;
     /* We can do a simple interpolation in the following cases:
      * We shrink by an integer value.
-     * If pixel_aspect<1 (e.g. NIKON D1X) shrink must be at least 4.
-     * Wanted size is smaller than raw size (size is after a raw->shrink).
-     * There are no filters (Foveon). */
-    uf->ConvertShrink = 1;
-    if ( uf->conf->interpolation==half_interpolation ||
-	 ( uf->conf->size==0 && uf->conf->shrink>1 ) ||
-	 ( uf->conf->size>0 &&
-	   uf->conf->size<=MAX(temp_height, temp_width) ) ||
-	 !uf->HaveFilters ) {
-	if (uf->conf->size==0 && uf->conf->shrink>1 &&
-		(int)(uf->conf->shrink*raw->pixel_aspect)%2==0)
-	    uf->ConvertShrink = uf->conf->shrink * raw->pixel_aspect;
-	else if (uf->conf->interpolation==half_interpolation)
-	    uf->ConvertShrink = 2;
-	else if ( uf->HaveFilters)
+     * If pixel_aspect<1 (e.g. NIKON D1X) shrink must be at least 4. */
+    if (uf->conf->size==0 && uf->conf->shrink>1) {
+	double pixel_aspect = MIN(raw->pixel_aspect, 1/raw->pixel_aspect);
+	uf->ConvertShrink = uf->conf->shrink * pixel_aspect;
+    } else if (uf->conf->interpolation==half_interpolation) {
+	uf->ConvertShrink = 2;
+    /* Wanted size is smaller than raw size (size is after a raw->shrink)
+     * (assuming there are filters). */
+    } else if (uf->conf->size>0 && uf->HaveFilters) {
+	int cropHeight = uf->conf->CropY2 - uf->conf->CropY1;
+	int cropWidth = uf->conf->CropX2 - uf->conf->CropX1;
+	int cropSize = MAX(cropHeight, cropWidth);
+	if (cropSize/uf->conf->size >= 2)
 	    uf->ConvertShrink = 2;
     }
     return UFRAW_SUCCESS;
@@ -732,19 +706,22 @@ int ufraw_convert_image_init(ufraw_data *uf)
 
 int ufraw_convert_image(ufraw_data *uf)
 {
+#ifdef UFRAW_HOTPIXELS
+    uf->mark_hotpixels = FALSE;
+#endif
     ufraw_developer_prepare(uf, file_developer);
     ufraw_convert_image_init(uf);
     ufraw_convert_image_first_phase(uf, TRUE);
     if (uf->ConvertShrink>1) {
-	dcraw_data *raw = uf->raw;
+	ufraw_image_data *FirstImage = &uf->Images[ufraw_first_phase];
 	dcraw_image_data final;
-	final.height = uf->image.height;
-	final.width = uf->image.width;
-	final.image = uf->image.image;
+	final.height = FirstImage->height;
+	final.width = FirstImage->width;
+	final.image = (image_type *)FirstImage->buffer;
 	/* Scale threshold according to shrink factor, as the effect of
 	 * neighbouring pixels decays about exponentially with distance. */
 	float threshold = uf->conf->threshold * exp(-(uf->ConvertShrink/2.0-1));
-	dcraw_wavelet_denoise_shrinked(&final, raw, threshold);
+	dcraw_wavelet_denoise_shrinked(&final, threshold);
     }
     return UFRAW_SUCCESS;
 }
@@ -921,31 +898,121 @@ no_distortion:
     }
 }
 
-#endif // HAVE_LENSFUN
+#endif /* HAVE_LENSFUN */
+
+#ifdef UFRAW_HOTPIXELS
+/*
+ * A pixel with a significantly larger value than all of its four direct
+ * neighbours is considered "hot". It will be replaced by the maximum value
+ * of its neighbours. For simplicity border pixels are not considered.
+ *
+ * Reasonable values for uf->conf->hotpixel are in the range 0.5-10.
+ *
+ * OpenMP notes
+ * ------------
+ * The hot pixel count is updated atomically but this is merely for proof
+ * of concept: objdump -Sd shows a "lock addl $0x1,0x1f4(%edx) sequence.
+ * The algorithm uses pixel values from previous and next row anyway and
+ * whether or not pixels are marked makes a difference already.
+ */
+static void ufraw_shave_hotpixels(ufraw_data *uf, dcraw_image_type *img,
+	int width, int height, int colors, unsigned rgbMax)
+{
+    int w, h, c, i;
+    unsigned delta, t, v, hi;
+    dcraw_image_type *p;
+
+    uf->hotpixels = 0;
+    if (uf->conf->hotpixel <= 0.0)
+	return;
+    delta = rgbMax / (uf->conf->hotpixel + 1.0);
+#ifdef _OPENMP
+#pragma omp parallel for schedule(static,64) default(none) \
+  shared(uf,img,width,height,colors,rgbMax,delta) \
+  private(h,p,w,c,t,v,hi,i)
+#endif
+    for (h = 1; h < height - 1; ++h) {
+	p = img + 1 + h * width;
+	for (w = 1; w < width - 1; ++w, ++p) {
+	    for (c = 0; c < colors; ++c) {
+		t = p[0][c];
+		if (t <= delta)
+		    continue;
+		t -= delta;
+		v = p[-1][c];
+		if (v > t)
+		    continue;
+		hi = v;
+		v = p[1][c];
+		if (v > t)
+		    continue;
+		if (v > hi)
+		    hi = v;
+		v = p[-w][c];
+		if (v > t)
+		    continue;
+		if (v > hi)
+		    hi = v;
+		v = p[w][c];
+		if (v > t)
+		    continue;
+		if (v > hi)
+		    hi = v;
+#if 0
+		if (uf->hotpixels < 100)
+		    printf("%u %u %u: %u->%u\t\n", w, h, c, p[0][c], hi);
+#endif
+		/* mark the pixel using the original hot value */
+		if (uf->mark_hotpixels) {
+		    for (i = -10; i >= -20 && w + i >= 0; --i)
+			memcpy(p[i], p[0], sizeof (p[i]));
+		    for (i = 10; i <= 20 && w + i < width; ++i)
+			memcpy(p[i], p[0], sizeof (p[i]));
+		}
+		p[0][c] = hi;
+#ifdef _OPENMP
+#pragma omp atomic
+#endif
+		++uf->hotpixels;
+	    }
+	}
+    }
+}
+#endif
 
 /* This is the part of the conversion which is not supported by
  * ufraw_convert_image_area() */
 int ufraw_convert_image_first_phase(ufraw_data *uf, gboolean lensfix)
 {
-    int status, c;
+    ufraw_image_data *FirstImage = &uf->Images[ufraw_first_phase];
     dcraw_data *raw = uf->raw;
+    dcraw_image_type *rawimage;
     // final->image memory will be realloc'd as needed
     dcraw_image_data final;
-    final.image = uf->image.image;
+    final.image = (image_type *)FirstImage->buffer;
+    final.width = FirstImage->width;
+    final.height = FirstImage->height;
     dcraw_data *dark = uf->conf->darkframe ? uf->conf->darkframe->raw : NULL;
 
+    rawimage = raw->raw.image;
+    raw->raw.image = g_memdup(rawimage, raw->raw.height * raw->raw.width *
+	    sizeof (dcraw_image_type));
+#ifdef UFRAW_HOTPIXELS
+    ufraw_shave_hotpixels(uf, raw->raw.image, raw->raw.width, raw->raw.height,
+	    raw->raw.colors, raw->rgbMax);
+#endif
     if ( uf->ConvertShrink>1 || !uf->HaveFilters ) {
 	dcraw_finalize_shrink(&final, raw, dark, uf->ConvertShrink);
+	uf->developer->doWB = 1;
     } else {
-	if ( (status=dcraw_wavelet_denoise(raw,
-		uf->conf->threshold))!=DCRAW_SUCCESS )
-	    return status;
-	dcraw_finalize_interpolate(&final, raw, dark, uf->conf->interpolation,
-		uf->conf->smoothing, uf->developer->rgbWB);
-	uf->developer->rgbMax = uf->developer->max;
-	for (c=0; c<4; c++)
-	    uf->developer->rgbWB[c] = 0x10000;
+	dcraw_wavelet_denoise(raw, uf->conf->threshold);
+	dcraw_finalize_interpolate(&final, raw, dark,
+	    uf->conf->interpolation, uf->conf->smoothing,
+	    uf->developer->rgbWB);
+	uf->developer->doWB = 0;
     }
+    g_free(raw->raw.image);
+    raw->raw.image = rawimage;
 
     dcraw_image_stretch(&final, raw->pixel_aspect);
     if (uf->conf->size==0 && uf->conf->shrink>1) {
@@ -963,7 +1030,7 @@ int ufraw_convert_image_first_phase(ufraw_data *uf, gboolean lensfix)
 	} else {
 	    /* uf->conf->size holds the size of the cropped image.
 	     * We need to calculate from it the desired size of
-	     * th uncropped image. */
+	     * the uncropped image. */
 	    int finalSize = uf->ConvertShrink * MAX(final.height, final.width);
 	    uf->ConvertShrink = cropSize / uf->conf->size;
 	    dcraw_image_resize(&final, uf->conf->size * finalSize / cropSize);
@@ -997,23 +1064,24 @@ int ufraw_convert_image_first_phase(ufraw_data *uf, gboolean lensfix)
     }
 #else
     (void)lensfix;
-#endif // HAVE_LENSFUN
+#endif /* HAVE_LENSFUN */
 
-    uf->image.image = final.image;
-    uf->image.height = final.height;
-    uf->image.width = final.width;
-
-    ufraw_image_data *FirstImage = &uf->Images[ufraw_first_phase];
-    FirstImage->height = uf->image.height;
-    FirstImage->width = uf->image.width;
+    FirstImage->height = final.height;
+    FirstImage->width = final.width;
     FirstImage->depth = sizeof(dcraw_image_type);
     FirstImage->rowstride = FirstImage->width * FirstImage->depth;
-    FirstImage->buffer = (guint8 *)uf->image.image;
+    FirstImage->buffer = (guint8 *)final.image;
     FirstImage->valid = 0xffffffff;
 
     return UFRAW_SUCCESS;
 }
 
+static gboolean ufraw_do_denoise_phase(ufraw_data *uf)
+{
+    // !doWB means denoise was done in first phase
+    return uf->conf->threshold>0 && uf->developer->doWB;
+}
+
 int ufraw_convert_image_init_phase(ufraw_data *uf)
 {
     ufraw_image_data *FirstImage = &uf->Images[ufraw_first_phase];
@@ -1021,8 +1089,7 @@ int ufraw_convert_image_init_phase(ufraw_data *uf)
 
     /* Denoise image layer */
     img = &uf->Images[ufraw_denoise_phase];
-    if (uf->conf->threshold > 0)
-    {
+    if (ufraw_do_denoise_phase(uf)) {
         /* Mark layer invalid if we're resizing */
         if (img->height != FirstImage->height ||
             img->width != FirstImage->width ||
@@ -1088,42 +1155,47 @@ int ufraw_convert_image_init_phase(ufraw_data *uf)
         uf->modifier = NULL;
         img->valid = 0;
     }
-#endif
+#endif /* HAVE_LENSFUN */
 
     return UFRAW_SUCCESS;
 }
 
-ufraw_image_data *ufraw_convert_image_area (
-    ufraw_data *uf, int saidx, UFRawPhase phase)
+ufraw_image_data *ufraw_convert_get_final_image(ufraw_data *uf)
+{
+#ifdef HAVE_LENSFUN
+    if (uf->modifier &&
+	(uf->postproc_ops & (LF_MODIFY_TCA | LF_MODIFY_DISTORTION |
+			     LF_MODIFY_GEOMETRY | LF_MODIFY_SCALE)))
+	return &uf->Images[ufraw_lensfun_phase];
+#endif /* HAVE_LENSFUN */
+    return &uf->Images[ufraw_develop_phase];
+}
+
+ufraw_image_data *ufraw_convert_image_area(ufraw_data *uf, unsigned saidx,
+	UFRawPhase phase)
 {
     int x = 0, y = 0, w = 0, h = 0;
     ufraw_image_data *out = &uf->Images [phase];
 
-    if (saidx >= 0)
-    {
-        if (out->valid & (1 << saidx))
-            return out; // the subarea has been already computed
-
-        /* Get subarea coordinates */
-        ufraw_img_get_subarea_coord (out, saidx, &x, &y, &w, &h);
-    }
+    if (out->valid & (1 << saidx))
+        return out; // the subarea has been already computed
 
     /* Get the subarea image for previous phase */
     ufraw_image_data *in = (phase > ufraw_first_phase) ?
         ufraw_convert_image_area (uf, saidx, phase - 1) :
         &uf->Images [ufraw_first_phase];
 
+    /* Get subarea coordinates */
+    ufraw_img_get_subarea_coord(out, saidx, &x, &y, &w, &h);
+
     switch (phase)
     {
         case ufraw_denoise_phase:
             {
-                if (uf->conf->threshold == 0)
+		if (!ufraw_do_denoise_phase(uf))
                     // No denoise phase, return the image from previous phase
                     return in;
 
-                if (saidx < 0)
-                    return out;
-
                 dcraw_image_data tmp;
                 /* With shrink==2 the border should be 16 pixels */
                 int border = 16 * 2 / uf->ConvertShrink;
@@ -1150,7 +1222,7 @@ ufraw_image_data *ufraw_convert_image_area (
                 /* Scale threshold according to shrink factor, as the effect of
                  * neighbouring pixels decays about exponentially with distance. */
                 float threshold = uf->conf->threshold * exp(1.0 - uf->ConvertShrink / 2.0);
-                dcraw_wavelet_denoise_shrinked (&tmp, uf->raw, threshold);
+                dcraw_wavelet_denoise_shrinked(&tmp, threshold);
                 for (yy = 0; yy < h; yy++)
                     memcpy (out->buffer + ((y + yy) * out->width + x) * out->depth,
                             tmp.image [(yy + y - by) * tmp.width + x - bx], w * out->depth);
@@ -1160,17 +1232,13 @@ ufraw_image_data *ufraw_convert_image_area (
 
         case ufraw_develop_phase:
             {
-                if (saidx < 0)
-                    return out;
-
-                guint16 *pixtmp = g_new (guint16, w * 3);
                 int yy;
                 for (yy = y; yy < y + h; yy++)
                 {
                     guint8 *dest = out->buffer + (yy * out->width + x) * out->depth;
-                    develope (
+                    develop(
                         dest, (void *)(in->buffer + (yy * in->width + x) * in->depth),
-                        uf->developer, 8, pixtmp, w);
+                        uf->developer, 8, w);
 
 #ifdef HAVE_LENSFUN
                     if (uf->modifier &&
@@ -1184,9 +1252,8 @@ ufraw_image_data *ufraw_convert_image_area (
                             LF_CR_3 (RED, GREEN, BLUE),
                             out->width * out->depth);
                     }
-#endif
+#endif /* HAVE_LENSFUN */
                 }
-                g_free (pixtmp);
             }
             break;
 
@@ -1198,9 +1265,6 @@ ufraw_image_data *ufraw_convert_image_area (
                                           LF_MODIFY_GEOMETRY | LF_MODIFY_SCALE)))
                     return in;
 
-                if (saidx < 0)
-                    return out;
-
                 int yy;
                 float *buff = g_new (float, (w < 8) ? 8 * 2 * 3 : w * 2 * 3);
 
@@ -1225,7 +1289,7 @@ ufraw_image_data *ufraw_convert_image_area (
                 for (yy = 0; yy < 8 * 2 * 3; yy += 2)
                 {
                     int idx = ufraw_img_get_subarea_idx (in, buff [yy], buff [yy + 1]);
-                    if (idx >= 0 && idx <= 31)
+                    if (idx <= 31)
                         ufraw_convert_image_area (uf, idx, phase - 1);
                 }
 
@@ -1277,14 +1341,15 @@ ufraw_image_data *ufraw_convert_image_area (
                 g_free (buff);
             }
             break;
-#else
-            // fallback
-#endif
+#endif /* HAVE_LENSFUN */
 
         default:
             return in;
     }
 
+#ifdef _OPENMP
+#pragma omp critical
+#endif
     // Mark the subarea as valid
     out->valid |= (1 << saidx);
 
@@ -1337,7 +1402,7 @@ static int ufraw_flip_image_buffer(ufraw_image_data *img, int flip)
     return UFRAW_SUCCESS;
 }
 
-void ufraw_flip_orientation(ufraw_data *uf, int flip)
+static void ufraw_flip_orientation(ufraw_data *uf, int flip)
 {
     const char flipMatrix[8][8] = {
 	{ 0, 1, 2, 3, 4, 5, 6, 7 }, /* No flip */
@@ -1352,14 +1417,83 @@ void ufraw_flip_orientation(ufraw_data *uf, int flip)
     uf->conf->orientation = flipMatrix[uf->conf->orientation][flip];
 }
 
+/*
+ * Normalize arbitrary rotations into a 0..90 degree range.
+ */
+void ufraw_normalize_rotation(ufraw_data *uf)
+{
+    int angle, flip = 0;
+
+    uf->conf->rotationAngle = fmod(uf->conf->rotationAngle, 360.0);
+    if (uf->conf->rotationAngle < 0.0)
+	uf->conf->rotationAngle += 360.0;
+    angle = floor(uf->conf->rotationAngle / 90) * 90;
+    switch (angle) {
+	case  90: flip = 6; break;
+	case 180: flip = 3; break;
+	case 270: flip = 5; break;
+    }
+    ufraw_flip_orientation(uf, flip);
+    uf->conf->rotationAngle -= angle;
+}
+
+/*
+ * Unnormalize a normalized rotaion into a -180..180 degree range,
+ * while orientation can be either 0 (normal) or 1 (flipped).
+ * All image processing code assumes normalized rotation, therefore
+ * each call to ufraw_unnormalize_rotation() must be followed by a call
+ * to ufraw_normalize_rotation().
+ */
+void ufraw_unnormalize_rotation(ufraw_data *uf)
+{
+    switch (uf->conf->orientation) {
+    case 5: /* Rotate 270 */
+	uf->conf->rotationAngle += 90;
+    case 3: /* Rotate 180 */
+	uf->conf->rotationAngle += 90;
+    case 6: /* Rotate 90 */
+	uf->conf->rotationAngle += 90;
+	uf->conf->orientation = 0;
+    case 0: /* No flip */
+	break;
+    case 4: /* Flip over diagonal "\" */
+	uf->conf->rotationAngle += 90;
+    case 2: /* Flip vertical */
+	uf->conf->rotationAngle += 90;
+    case 7: /* Flip over diagonal "/" */
+	uf->conf->rotationAngle += 90;
+	uf->conf->orientation = 1;
+    case 1: /* Flip horizontal */
+	break;
+    default:
+	g_error("ufraw_unnormalized_roation(): orientation=%d out of range",
+		uf->conf->orientation);
+    }
+    uf->conf->rotationAngle = remainder(uf->conf->rotationAngle, 360.0);
+}
+
 int ufraw_flip_image(ufraw_data *uf, int flip)
 {
     ufraw_flip_orientation(uf, flip);
-
+    /* Usually orientation is applied before rotationAngle.
+     * Here we are flipping after rotationAngle was applied.
+     * We need to correct rotationAngle for this since these
+     * operations do no commute.
+     */
+    if (flip==1 || flip==2 || flip==4 || flip==7) {
+	uf->conf->rotationAngle = -uf->conf->rotationAngle;
+	ufraw_normalize_rotation(uf);
+    }
     ufraw_flip_image_buffer(&uf->Images[ufraw_first_phase], flip);
-    if ( uf->conf->threshold>0 )
+    if (ufraw_do_denoise_phase(uf))
 	ufraw_flip_image_buffer(&uf->Images[ufraw_denoise_phase], flip);
     ufraw_flip_image_buffer(&uf->Images[ufraw_develop_phase], flip);
+#ifdef HAVE_LENSFUN
+    if (uf->modifier &&
+	(uf->postproc_ops & (LF_MODIFY_TCA | LF_MODIFY_DISTORTION |
+			     LF_MODIFY_GEOMETRY | LF_MODIFY_SCALE)))
+	ufraw_flip_image_buffer(&uf->Images[ufraw_lensfun_phase], flip);
+#endif /* HAVE_LENSFUN */
 
     return UFRAW_SUCCESS;
 }
@@ -1370,6 +1504,11 @@ int ufraw_set_wb(ufraw_data *uf)
     double rgbWB[3];
     int status, c, cc, i;
 
+    UFRawPhase phase = uf->developer!=NULL && uf->developer->doWB ?
+	    ufraw_develop_phase : ufraw_first_phase;
+    for (; phase < ufraw_phases_num; phase++)
+	uf->Images[phase].valid = 0;
+
     /* For manual_wb we calculate chanMul from the temperature/green. */
     /* For all other it is the other way around. */
     if (!strcmp(uf->conf->wb, manual_wb)) {
@@ -1593,7 +1732,7 @@ void ufraw_auto_expose(ufraw_data *uf)
 {
     int sum, stop, wp, c, pMax, pMin, p;
     image_type pix;
-    guint16 p16[3], pixtmp[3];
+    guint16 p16[3];
 
     if (uf->conf->autoExposure!=apply_state) return;
 
@@ -1610,7 +1749,7 @@ void ufraw_auto_expose(ufraw_data *uf)
     {
 	for (c=0; c<uf->colors; c++)
 	    pix[c] = MIN (p * maxChan/uf->conf->chanMul[c], uf->rgbMax);
-	develope(p16, pix, uf->AutoDeveloper, 16, pixtmp, 1);
+	develop(p16, pix, uf->AutoDeveloper, 16, 1);
 	for (c=0, wp=0; c<3; c++) wp = MAX(wp, p16[c]);
 	if (wp < 0x10000 * 99/100) pMin = p;
 	else pMax = p;
@@ -1637,7 +1776,7 @@ void ufraw_auto_black(ufraw_data *uf)
 {
     int sum, stop, bp, c;
     image_type pix;
-    guint16 p16[3], pixtmp[3];
+    guint16 p16[3];
 
     if (uf->conf->autoBlack==disabled_state) return;
 
@@ -1652,7 +1791,7 @@ void ufraw_auto_black(ufraw_data *uf)
     for (c=0; c<uf->colors; c++) maxChan = MAX(uf->conf->chanMul[c], maxChan);
     for (c=0; c<uf->colors; c++)
 	pix[c] = MIN (bp * maxChan/uf->conf->chanMul[c], uf->rgbMax);
-    develope(p16, pix, uf->AutoDeveloper, 16, pixtmp, 1);
+    develop(p16, pix, uf->AutoDeveloper, 16, 1);
     for (c=0, bp=0; c<3; c++) bp = MAX(bp, p16[c]);
 
     CurveDataSetPoint(&uf->conf->curve[uf->conf->curveIndex],
@@ -1670,7 +1809,7 @@ void ufraw_auto_curve(ufraw_data *uf)
 {
     int sum, stop, steps=8, bp, p, i, j, c;
     image_type pix;
-    guint16 p16[3], pixtmp[3];
+    guint16 p16[3];
     CurveData *curve = &uf->conf->curve[uf->conf->curveIndex];
     double decay = 0.90;
     double norm = (1-pow(decay,steps))/(1-decay);
@@ -1687,7 +1826,7 @@ void ufraw_auto_curve(ufraw_data *uf)
 	    sum += uf->RawHistogram[bp];
 	for (c=0; c<uf->colors; c++)
 	    pix[c] = MIN (bp * maxChan/uf->conf->chanMul[c], uf->rgbMax);
-	develope(p16, pix, uf->AutoDeveloper, 16, pixtmp, 1);
+	develop(p16, pix, uf->AutoDeveloper, 16, 1);
 	for (c=0, p=0; c<3; c++) p = MAX(p, p16[c]);
 	stop += uf->RawCount * pow(decay,i) / norm;
 	/* Skip adding point if slope is too big (more than 4) */
@@ -1716,7 +1855,7 @@ void ufraw_auto_curve(ufraw_data *uf)
     }
 }
 
-void ufraw_rotate_row(image_data *image, void *pixbuf, double angle,
+void ufraw_rotate_row(ufraw_image_data *image, void *pixbuf, double angle,
 		      int bitDepth, int row, int offset, int width)
 {
     int col, ur, uc, i, j, in_image;
@@ -1770,3 +1909,71 @@ void ufraw_rotate_row(image_data *image, void *pixbuf, double angle,
 	}
     }
 }
+
+/*
+ * Rotate an 8 or 16 bits per color image 0-90 degrees. The output pixel is
+ * calculated as a weighted average of the 4 nearest input pixels.
+ */
+void ufraw_rotate_image_buffer(ufraw_image_data *img, double angle)
+{
+    double sine, cosine, movecol, moverow;
+    int oldwidth, oldheight, oldrowstride;
+    int width, height, depth, rowstride;
+    int col, row, uc, ur, i;
+    float c, r, fc, fr;
+    guint8 *in, *in_00, *in_01, *in_10, *in_11, *out;
+
+    if (!angle)
+	return;
+    sine = sin(angle * 2 * M_PI / 360);
+    cosine = cos(angle * 2 * M_PI / 360);
+    depth = img->depth;
+    oldwidth = img->width;
+    oldheight = img->height;
+    oldrowstride = img->rowstride;
+    width = ceil((oldheight * sine) + (oldwidth * cosine));
+    height = ceil((oldwidth * sine) + (oldheight * cosine));
+    rowstride = width * depth;
+    in = img->buffer;
+    out = g_new0(guint8, height * rowstride);
+    img->buffer = out;
+    img->width = width;
+    img->height = height;
+    img->rowstride = rowstride;
+    movecol = -oldheight * sine * cosine;
+    moverow = oldheight * sine * sine;
+    for (row = 0; row < height; ++row) {
+	for (col = 0; col < width; col++) {
+	    // (c,r) are the float input subpixel coordinates. fc,fr are the
+	    // weighting factors for integer coordinates (uc,ur)..(uc+1,ur+1)
+	    // around (c,r)
+	    c = (col * cosine) + (row * sine) + movecol;
+	    r = (row * cosine) - (col * sine) + moverow;
+	    uc = (int)floor(c);
+	    ur = (int)floor(r);
+	    if (uc >= 0 && uc + 1 < oldwidth && ur >= 0 && ur + 1 < oldheight) {
+		fc = c - uc;
+		fr = r - ur;
+		in_00 = in + ((ur * oldwidth) + uc) * depth;	// (uc,ur)
+		in_01 = in_00 + depth;				// (uc+1,ur)
+		in_10 = in_00 + oldrowstride;			// (uc,ur+1)
+		in_11 = in_10 + depth;				// (uc+1,ur+1)
+		if (depth > 4) {
+		    for (i = 0; i < 3; ++i) {
+			((guint16 *)out)[i] =
+			    (((guint16 *)in_00)[i] * (1 - fc) + ((guint16 *)in_01)[i] * fc) * (1 - fr) +
+			    (((guint16 *)in_10)[i] * (1 - fc) + ((guint16 *)in_11)[i] * fc) * fr;
+		    }
+		} else {
+		    for (i = 0; i < 3; ++i) {
+			out[i] =
+			    (in_00[i] * (1 - fc) + in_01[i] * fc) * (1 - fr) +
+			    (in_10[i] * (1 - fc) + in_11[i] * fc) * fr;
+		    }
+		}
+	    }
+	    out += depth;
+	}
+    }
+    g_free(in);
+}
