@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_ui.h - Common definitions for UFRaw's GUI.
- * Copyright 2004-2008 by Udi Fuchs
+ * Copyright 2004-2009 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
 
 #define raw_his_size 320
 #define live_his_size 256
-#define min_scale 2
+#define min_scale 1
 #define max_scale 20
 
 typedef struct {
@@ -61,12 +61,18 @@ typedef struct {
     GtkTable *GrayscaleMixerTable;
     GtkLabel *GrayscaleMixerColor;
     GtkLabel *SpotPatch;
+#ifdef UFRAW_HOTPIXELS
+    GtkLabel *HotpixelCount;
+#endif
     colorLabels *SpotLabels, *AvrLabels, *DevLabels, *OverLabels, *UnderLabels;
     GtkToggleButton *AutoExposureButton, *AutoBlackButton, *LockAspectButton;
-    GtkButton *AutoCurveButton;
+    GtkWidget *AutoCurveButton;
     GtkWidget *ResetWBButton, *ResetGammaButton, *ResetLinearButton;
     GtkWidget *ResetExposureButton, *ResetSaturationButton;
     GtkWidget *ResetThresholdButton;
+#ifdef UFRAW_HOTPIXELS
+    GtkWidget *ResetHotpixelButton;
+#endif
 #ifdef UFRAW_CONTRAST
     GtkWidget *ResetContrastButton;
 #endif
@@ -92,10 +98,19 @@ typedef struct {
     GtkAdjustment *LinearAdjustment;
     GtkAdjustment *ExposureAdjustment;
     GtkAdjustment *ThresholdAdjustment;
+#ifdef UFRAW_HOTPIXELS
+    GtkAdjustment *HotpixelAdjustment;
+#endif
     GtkAdjustment *SaturationAdjustment;
 #ifdef UFRAW_CONTRAST
     GtkAdjustment *ContrastAdjustment;
 #endif
+    GtkWidget *LightnessHueSelectNewButton;
+    GtkTable *LightnessAdjustmentTable[max_adjustments];
+    GtkAdjustment *LightnessAdjustment[max_adjustments];
+    GtkWidget *ResetLightnessAdjustmentButton[max_adjustments];
+    GtkWidget *LightnessHueSelectButton[max_adjustments];
+    GtkWidget *LightnessHueRemoveButton[max_adjustments];
     GtkAdjustment *CropX1Adjustment;
     GtkAdjustment *CropY1Adjustment;
     GtkAdjustment *CropX2Adjustment;
@@ -104,6 +119,8 @@ typedef struct {
     GtkAdjustment *ShrinkAdjustment;
     GtkAdjustment *HeightAdjustment;
     GtkAdjustment *WidthAdjustment;
+    GtkAdjustment *RotationAdjustment;
+    GtkWidget *ResetRotationAdjustment;
     GtkAdjustment *GrayscaleMixers[3];
 #ifdef HAVE_LENSFUN
     /* The GtkEntry with camera maker/model name */
@@ -138,7 +155,7 @@ typedef struct {
     GtkWidget *LensScaleResetButton;
     /* The button that automatically computes the optimal scale */
     GtkWidget *LensAutoScaleButton;
-#endif
+#endif /* HAVE_LENSFUN */
     long (*SaveFunc)();
     RenderModeType RenderMode;
     /* Current subarea index (0-31). If negative, rendering has stopped */
@@ -157,6 +174,7 @@ typedef struct {
     int PageNum;
     int PageNumSpot;
     int PageNumGray;
+    int PageNumLightness;
     int PageNumCrop;
     int HisMinHeight;
     /* Original aspect ratio (0) or actual aspect ratio */
@@ -196,9 +214,16 @@ GtkWidget *table_with_frame (GtkWidget *box, char *label, gboolean expand);
 
 GtkWidget *notebook_page_new (GtkNotebook *notebook, char *text, char *icon);
 
-GtkAdjustment *adjustment_scale (
-    GtkTable *table, int x, int y, const char *label, double value, void *valuep,
-    double min, double max, double step, double jump, long accuracy, const char *tip,
-    GCallback callback);
+GtkWidget *stock_image_button(const gchar *stock_id, GtkIconSize size,
+			      const char *tip, GCallback callback, void *data);
+GtkWidget *stock_icon_button(const gchar *stock_id,
+			     const char *tip, GCallback callback, void *data);
+GtkWidget *reset_button (const char *tip, GCallback callback, void *data);
+
+GtkAdjustment *adjustment_scale (GtkTable *table, int x, int y,
+    const char *label, double value, void *valuep, double min, double max,
+    double step, double jump, long accuracy, const gboolean wrap_spinner,
+    const char *tip, GCallback callback, GtkWidget **resetButton,
+    const char *resetTip, void (*resetCallback)());
 
 #endif /* _UFRAW_UI_H */
