@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_writer.c - functions to output image files in different formats.
- * Copyright 2004-2008 by Udi Fuchs
+ * Copyright 2004-2009 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
 
 #include <errno.h>	/* for errno */
 #include <string.h>
+#include <lcms.h>
 #include "uf_glib.h"
 #include <glib/gi18n.h>
 #ifdef HAVE_LIBTIFF
@@ -61,42 +62,26 @@ static void grayscale_buffer(void *graybuf, int width, int bitDepth)
     }
 }
 
-static int ppm8_row_writer(
-    ufraw_data *uf,
-    void * volatile out,
-    void * pixbuf,
-    int row, int width, int grayscale)
+static int ppm_row_writer(ufraw_data *uf, void *volatile out, void *pixbuf,
+    int row, int width, int height, int grayscale, int bitDepth)
 {
-    row=row;
-    if ((int)fwrite(pixbuf, grayscale ? 1 : 3, width, out)<width) {
-	ufraw_set_error(uf, _("Error creating file '%s'."),
-        uf->conf->outputFilename);
-	ufraw_set_error(uf, g_strerror(errno));
-	return UFRAW_ERROR;
-    } else {
-	return UFRAW_SUCCESS;
-    }
-}
-
-static int ppm16_row_writer(
-    ufraw_data *uf,
-    void * volatile out,
-    void * pixbuf,
-    int row, int width, int grayscale)
-{
-    row=row;
-    guint16 *pixbuf16 = (guint16 *) pixbuf;
+    (void)row;
+    int rowStride = width * (grayscale ? 1 : 3) * (bitDepth>8 ? 2 : 1);
     int i;
-    for (i=0; i<3*width; i++)
-	pixbuf16[i] = g_htons(pixbuf16[i]);
-    if ((int)fwrite(pixbuf16, grayscale ? 2 : 6, width, out)<width) {
-	ufraw_set_error(uf, _("Error creating file '%s'."),
-	uf->conf->outputFilename);
-	ufraw_set_error(uf, g_strerror(errno));
-	return UFRAW_ERROR;
-    } else {
-	return UFRAW_SUCCESS;
+    if ( bitDepth>8 ) {
+	guint16 *pixbuf16 = (guint16 *)pixbuf;
+	for (i=0; i<3*width*height; i++)
+	    pixbuf16[i] = g_htons(pixbuf16[i]);
     }
+    for (i=0; i<height; i++) {
+	if ((int)fwrite(pixbuf+i*width*(bitDepth>8?6:3), rowStride, 1, out)<1) {
+	    ufraw_set_error(uf, _("Error creating file '%s'."),
+            uf->conf->outputFilename);
+	    ufraw_set_error(uf, g_strerror(errno));
+	    return UFRAW_ERROR;
+	}
+    }
+    return UFRAW_SUCCESS;
 }
 
 #ifdef HAVE_LIBTIFF
@@ -110,22 +95,22 @@ static void tiff_messenger(const char *module, const char *fmt, va_list ap)
     vsnprintf(ufraw_tiff_message, max_path, fmt, ap);
 }
 
-int tiff_row_writer(
-    ufraw_data *uf,
-    void * volatile out,
-    void * pixbuf,
-    int row, int width, int grayscale)
+int tiff_row_writer(ufraw_data *uf, void *volatile out, void *pixbuf,
+    int row, int width, int height, int grayscale, int bitDepth)
 {
-    width=width; grayscale=grayscale;
-    if (TIFFWriteScanline(out, pixbuf, row, 0)<0) {
-	// 'errno' does seem to contain useful information
-	ufraw_set_error(uf, _("Error creating file."));
-	ufraw_set_error(uf, ufraw_tiff_message);
-	ufraw_tiff_message[0] = '\0';
-	return UFRAW_ERROR;
-    } else {
-	return UFRAW_SUCCESS;
+    (void)grayscale;
+    int rowStride = width*( bitDepth>8 ? 6 : 3);
+    int i;
+    for (i=0; i<height; i++) {
+	if (TIFFWriteScanline(out, pixbuf+i*rowStride, row+i, 0)<0) {
+	    // 'errno' does seem to contain useful information
+	    ufraw_set_error(uf, _("Error creating file."));
+	    ufraw_set_error(uf, ufraw_tiff_message);
+	    ufraw_tiff_message[0] = '\0';
+	    return UFRAW_ERROR;
+	}
     }
+    return UFRAW_SUCCESS;
 }
 #endif /*HAVE_LIBTIFF*/
 
@@ -164,19 +149,20 @@ static void jpeg_error_handler(j_common_ptr cinfo)
 	    cinfo->err->msg_parm.i[3]);
 }
 
-static int jpeg_row_writer(
-    ufraw_data *uf,
-    void * volatile out,
-    void * pixbuf,
-    int row, int width, int grayscale)
+static int jpeg_row_writer(ufraw_data *uf, void *volatile out, void *pixbuf,
+    int row, int width, int height, int grayscale, int bitDepth)
 {
-    row=row; width=width; grayscale=grayscale;
-    guint8 *pixbuf8 = pixbuf;
-    jpeg_write_scanlines((struct jpeg_compress_struct *)out, &pixbuf8, 1);
-    if ( ufraw_is_error(uf) )
-	return UFRAW_ERROR;
-    else
-	return UFRAW_SUCCESS;
+    (void)row;
+    (void)grayscale;
+    (void)bitDepth;
+    int i;
+    for (i=0; i<height; i++) {
+	guint8 *pixbuf8 = pixbuf + 3*width*i;
+	jpeg_write_scanlines((struct jpeg_compress_struct *)out, &pixbuf8, 1);
+	if ( ufraw_is_error(uf) )
+	    return UFRAW_ERROR;
+    }
+    return UFRAW_SUCCESS;
 }
 #endif /*HAVE_LIBJPEG*/
 
@@ -200,37 +186,39 @@ static void PNGwriteRawProfile(png_struct *ping,
     png_info *ping_info, char *profile_type, guint8 *profile_data,
     png_uint_32 length);
 
-int png_row_writer(
-    ufraw_data *uf,
-    void * volatile out,
-    void * pixbuf,
-    int row, int width, int grayscale)
+int png_row_writer(ufraw_data *uf, void *volatile out, void *pixbuf,
+    int row, int width, int height, int grayscale, int bitDepth)
 {
-    uf=uf; row=row; width=width; grayscale=grayscale;
+    (void)uf;
+    (void)row;
+    (void)grayscale;
+    int rowStride = width * (bitDepth>8 ? 6 : 3);
 
-    png_write_row(out, (guint8 *)pixbuf);
+    int i;
+    for (i=0; i<height; i++)
+	png_write_row(out, (guint8 *)pixbuf + rowStride*i);
 
     return UFRAW_SUCCESS;
 }
 #endif /*HAVE_LIBPNG*/
 
-void write_image_data(
+void ufraw_write_image_data(
     ufraw_data *uf,
     void * volatile out,
     int width, int height, int left, int top, int bitDepth, int grayscaleMode,
-    int (*row_writer) (ufraw_data *, void * volatile, void *, int, int, int))
+    int (*row_writer) (ufraw_data *, void * volatile, void *, int, int, int, int, int))
 {
-    int row, row0, rowStride;
-    image_type *rawImage;
-    rowStride = uf->image.width;
-    rawImage = uf->image.image;
-    guint16 pixbuf[rowStride * 3 * DEVELOP_BATCH];
+    int row, row0;
+    int rowStride = uf->Images[ufraw_first_phase].width;
+    image_type *rawImage = (image_type *)uf->Images[ufraw_first_phase].buffer;
+    int byteDepth = (bitDepth+7)/8;
+    guint8 pixbuf8[width * 3 * byteDepth * DEVELOP_BATCH];
 
     if (uf->conf->rotationAngle != 0) {
 	// Buffer for unrotated image.
-	image_data image;
-	image.width = uf->image.width;
-	image.height = uf->image.height;
+	ufraw_image_data image;
+	image.width = uf->Images[ufraw_first_phase].width;
+	image.height = uf->Images[ufraw_first_phase].height;
 	image.depth = 6;
 	image.rowstride = image.width * image.depth;
 	image.buffer = g_new(guint8, image.height * image.rowstride);
@@ -243,8 +231,8 @@ void write_image_data(
 		preview_progress(uf->widget, _("Converting image"),
 		    0.5 * row0/image.height);
 	    guint8 *rowbuf = &image.buffer[row0*image.rowstride];
-	    develope(rowbuf, rawImage[row0*rowStride],
-		     uf->developer, 16, (guint16*)rowbuf,
+	    develop(rowbuf, rawImage[row0*rowStride],
+		     uf->developer, 16,
 		     MIN(image.height - row0, DEVELOP_BATCH) * image.width);
 	}
 	// Write rotated image to output.
@@ -257,42 +245,39 @@ void write_image_data(
 	    for (row = 0; row < DEVELOP_BATCH; row++) {
 		if (row + row0 >= height)
 		    continue;
-		guint16 *rowbuf = &pixbuf[row * rowStride * 3];
+		guint8 *rowbuf = &pixbuf8[row * width * 3 * byteDepth];
 		ufraw_rotate_row(&image, rowbuf, uf->conf->rotationAngle,
 		    bitDepth, top+row+row0, left, width);
 		if (grayscaleMode)
 		    grayscale_buffer(rowbuf, width, bitDepth);
 	    }
-	    for (row = row0; row < height && row < row0 + DEVELOP_BATCH; row++) {
-		guint16 *rowbuf = &pixbuf[(row-row0) * rowStride * 3];
-		if (row_writer(uf, out, rowbuf, row, width, grayscaleMode) != UFRAW_SUCCESS)
-		    break;
-	    }
+	    int batchHeight = MIN(height-row0, DEVELOP_BATCH);
+	    if ( row_writer(uf, out, pixbuf8, row0, width, batchHeight,
+		    grayscaleMode, bitDepth) != UFRAW_SUCCESS )
+		break;
 	}
 	g_free(image.buffer);
     } else {
-	guint16 *rowbuf;
 	// No rotation required. Develop straight to output.
 	for (row0 = 0; row0 < height; row0 += DEVELOP_BATCH) {
 	    preview_progress(uf->widget, _("Saving image"),
 		0.5 + 0.5*row0/height);
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) private(row, rowbuf)
+#pragma omp parallel for default(shared) private(row)
 #endif
 	    for (row = 0; row < DEVELOP_BATCH; row++) {
 		if (row + row0 >= height)
 		    continue;
-		rowbuf = &pixbuf[row * rowStride * 3];
-		develope(rowbuf, rawImage[(top+row+row0)*rowStride+left],
-	            uf->developer, bitDepth, rowbuf, width);
+		guint8 *rowbuf = &pixbuf8[row * width * 3 * byteDepth];
+		develop(rowbuf, rawImage[(top+row+row0)*rowStride+left],
+	            uf->developer, bitDepth, width);
 		if (grayscaleMode)
 		    grayscale_buffer(rowbuf, width, bitDepth);
 	    }
-	    for (row = row0; row < height && row < row0 + DEVELOP_BATCH; row++) {
-		rowbuf = &pixbuf[(row-row0) * rowStride * 3];
-		if (row_writer(uf, out, rowbuf, row, width, grayscaleMode) != UFRAW_SUCCESS)
-		    break;
-	    }
+	    int batchHeight = MIN(height-row0, DEVELOP_BATCH);
+	    if ( row_writer(uf, out, pixbuf8, row0, width, batchHeight,
+		    grayscaleMode, bitDepth) != UFRAW_SUCCESS )
+		break;
 	}
     }
 }
@@ -390,25 +375,26 @@ int ufraw_write_image(ufraw_data *uf)
     }
     // TODO: error handling
     ufraw_convert_image(uf);
-    left = uf->conf->CropX1 * uf->image.width / uf->rotatedWidth;
-    top = uf->conf->CropY1 * uf->image.height / uf->rotatedHeight;
+    ufraw_image_data *FirstImage = &uf->Images[ufraw_first_phase];
+    left = uf->conf->CropX1 * FirstImage->width / uf->initialWidth;
+    top = uf->conf->CropY1 * FirstImage->height / uf->initialHeight;
     volatile int BitDepth = uf->conf->profile[out_profile]
 			[uf->conf->profileIndex[out_profile]].BitDepth;
     if ( BitDepth!=16 ) BitDepth = 8;
     width = (uf->conf->CropX2 - uf->conf->CropX1)
-	    * uf->image.width / uf->initialWidth;
+	    * FirstImage->width / uf->initialWidth;
     height = (uf->conf->CropY2 - uf->conf->CropY1)
-	    * uf->image.height / uf->initialHeight;
+	    * FirstImage->height / uf->initialHeight;
     if ( uf->conf->type==ppm_type && BitDepth==8 ) {
 	fprintf(out, "P%c\n%d %d\n%d\n",
 		grayscaleMode ? '5' : '6', width, height, 0xFF);
-	write_image_data(uf, out, width, height, left, top,
-			 BitDepth, grayscaleMode, ppm8_row_writer);
+	ufraw_write_image_data(uf, out, width, height, left, top,
+			 BitDepth, grayscaleMode, ppm_row_writer);
     } else if ( uf->conf->type==ppm_type && BitDepth==16 ) {
 	fprintf(out, "P%c\n%d %d\n%d\n",
 		grayscaleMode ? '5' : '6', width, height, 0xFFFF);
-	write_image_data(uf, out, width, height, left, top,
-			 BitDepth, grayscaleMode, ppm16_row_writer);
+	ufraw_write_image_data(uf, out, width, height, left, top,
+			 BitDepth, grayscaleMode, ppm_row_writer);
 #ifdef HAVE_LIBTIFF
     } else if ( uf->conf->type==tiff_type ) {
 	TIFFSetField(out, TIFFTAG_IMAGEWIDTH, width);
@@ -428,7 +414,7 @@ int ufraw_write_image(ufraw_data *uf)
 	else
 #endif
 	    TIFFSetField(out, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
-	/* Embed output profile if it is not the internal sRGB*/
+	/* Embed output profile if it is not the internal sRGB. */
 	if (strcmp(uf->developer->profileFile[out_profile], "")) {
 	    char *buf;
 	    gsize len;
@@ -442,10 +428,26 @@ int ufraw_write_image(ufraw_data *uf)
 			uf->developer->profileFile[out_profile],
 			uf->conf->outputFilename);
 	    }
+	} else if (uf->conf->profileIndex[out_profile]==1) { // Embed sRGB.
+	    cmsHPROFILE hOutProfile = cmsCreate_sRGBProfile();
+	    gsize len = 0;
+	    _cmsSaveProfileToMem(hOutProfile, 0, &len); // Calculate len.
+	    if (len > 0) {
+		unsigned char buf[len];
+		_cmsSaveProfileToMem(hOutProfile, buf, &len);
+		TIFFSetField(out, TIFFTAG_ICCPROFILE, len, buf);
+	    } else {
+                ufraw_set_warning(uf,
+                    _("Failed to embed output profile '%s' in '%s'."),
+		    uf->conf->profile[out_profile]
+			    [uf->conf->profileIndex[out_profile]].name,
+                    uf->conf->outputFilename);
+	    }
+	    cmsCloseProfile(hOutProfile);
 	}
 	TIFFSetField(out, TIFFTAG_ROWSPERSTRIP, TIFFDefaultStripSize(out, 0));
 
-	write_image_data(uf, out, width, height, left, top,
+	ufraw_write_image_data(uf, out, width, height, left, top,
 			 BitDepth, grayscaleMode, tiff_row_writer);
 
 #endif /*HAVE_LIBTIFF*/
@@ -486,7 +488,7 @@ int ufraw_write_image(ufraw_data *uf)
 
 	jpeg_start_compress(&cinfo, TRUE);
 
-	/* Embed output profile if it is not the internal sRGB*/
+	/* Embed output profile if it is not the internal sRGB. */
 	if (strcmp(uf->developer->profileFile[out_profile], "")) {
 	    char *buf;
 	    gsize len;
@@ -500,6 +502,22 @@ int ufraw_write_image(ufraw_data *uf)
 			uf->developer->profileFile[out_profile],
 			uf->conf->outputFilename);
 	    }
+	} else if (uf->conf->profileIndex[out_profile]==1) { // Embed sRGB.
+	    cmsHPROFILE hOutProfile = cmsCreate_sRGBProfile();
+	    gsize len = 0;
+	    _cmsSaveProfileToMem(hOutProfile, 0, &len); // Calculate len.
+	    if (len > 0) {
+		unsigned char buf[len];
+		_cmsSaveProfileToMem(hOutProfile, buf, &len);
+		write_icc_profile(&cinfo, buf, len);
+	    } else {
+                ufraw_set_warning(uf,
+                    _("Failed to embed output profile '%s' in '%s'."),
+		    uf->conf->profile[out_profile]
+			    [uf->conf->profileIndex[out_profile]].name,
+                    uf->conf->outputFilename);
+	    }
+	    cmsCloseProfile(hOutProfile);
 	}
 	if ( uf->conf->embedExif ) {
 	    ufraw_exif_prepare_output(uf);
@@ -515,7 +533,7 @@ int ufraw_write_image(ufraw_data *uf)
 	    }
 	}
 
-	write_image_data(uf, &cinfo, width, height, left, top,
+	ufraw_write_image_data(uf, &cinfo, width, height, left, top,
 			 8, grayscaleMode, jpeg_row_writer);
 
 	if ( ufraw_is_error(uf) ) {
@@ -559,7 +577,7 @@ int ufraw_write_image(ufraw_data *uf)
 		    uf->conf->make, uf->conf->model);
 	    png_set_text(png, info, text, 2);
 	    g_free(text[1].text);
-	    /* Embed output profile if it is not the internal sRGB*/
+	    /* Embed output profile if it is not the internal sRGB. */
 	    if (strcmp(uf->developer->profileFile[out_profile], "")) {
 		char *buf;
 		gsize len;
@@ -576,6 +594,26 @@ int ufraw_write_image(ufraw_data *uf)
 			uf->developer->profileFile[out_profile],
 			uf->conf->outputFilename);
 		}
+	    } else if (uf->conf->profileIndex[out_profile]==1) { // Embed sRGB.
+		cmsHPROFILE hOutProfile = cmsCreate_sRGBProfile();
+		gsize len = 0;
+		_cmsSaveProfileToMem(hOutProfile, 0, &len); // Calculate len.
+		if (len > 0) {
+		    char buf[len];
+		    _cmsSaveProfileToMem(hOutProfile, buf, &len);
+		    png_set_iCCP(png, info,
+			    uf->conf->profile[out_profile]
+				    [uf->conf->profileIndex[out_profile]].name,
+			    PNG_COMPRESSION_TYPE_BASE,
+			    buf, len);
+		} else {
+		    ufraw_set_warning(uf,
+			_("Failed to embed output profile '%s' in '%s'."),
+			uf->conf->profile[out_profile]
+				[uf->conf->profileIndex[out_profile]].name,
+			uf->conf->outputFilename);
+		}
+		cmsCloseProfile(hOutProfile);
 	    }
 	    if ( uf->conf->embedExif ) {
 		ufraw_exif_prepare_output(uf);
@@ -587,7 +625,7 @@ int ufraw_write_image(ufraw_data *uf)
 	    if (BitDepth != 8 && G_BYTE_ORDER==G_LITTLE_ENDIAN )
 		png_set_swap(png); // Swap byte order to big-endian
 
-	    write_image_data(uf, png, width, height, left, top,
+	    ufraw_write_image_data(uf, png, width, height, left, top,
 			     BitDepth, grayscaleMode, png_row_writer);
 
 	    png_write_end(png, NULL);
@@ -615,10 +653,14 @@ int ufraw_write_image(ufraw_data *uf)
 
 	int row;
 	int i;
-	image_type *rawImage = uf->image.image;
-	int rowStride = uf->image.width;
+	image_type *rawImage =
+		(image_type *)uf->Images[ufraw_first_phase].buffer;
+	int rowStride = uf->Images[ufraw_first_phase].width;
 	guint16 pixbuf16[3];
 
+	// Avoid FITS images being saved upside down
+	ufraw_flip_image(uf, 2);
+
 	for (row=0; row<height; row++) {
 	    if (row%100==99)
 		preview_progress(uf->widget, _("Saving image"),
@@ -856,7 +898,3 @@ static void PNGwriteRawProfile(png_struct *ping,
     png_free(ping, text);
 }
 #endif /*HAVE_LIBPNG*/
-
-/* vim:set shiftwidth=4: */
-/* vim:set softtabstop=4: */
-/* emacs: -*- c-basic-offset: 4 -*- */
